# coding: utf8
# cython: infer_types=True
# cython: bounds_check=False
# cython: profile=True
from __future__ import unicode_literals

cimport cython
cimport numpy as np
import numpy
import numpy.linalg
import struct
import dill
import msgpack
from thinc.neural.util import get_array_module, copy_array

from libc.string cimport memcpy, memset
from libc.math cimport sqrt

from .span cimport Span
from .token cimport Token
from .span cimport Span
from .token cimport Token
from .printers import parse_tree
from ..lexeme cimport Lexeme, EMPTY_LEXEME
from ..typedefs cimport attr_t, flags_t
from ..attrs import intify_attrs, IDS
from ..attrs cimport attr_id_t
from ..attrs cimport ID, ORTH, NORM, LOWER, SHAPE, PREFIX, SUFFIX, CLUSTER
from ..attrs cimport LENGTH, POS, LEMMA, TAG, DEP, HEAD, SPACY, ENT_IOB
from ..attrs cimport ENT_TYPE, SENT_START
from ..parts_of_speech cimport CCONJ, PUNCT, NOUN, univ_pos_t
from ..util import normalize_slice
from ..compat import is_config, copy_reg, pickle, basestring_
from ..errors import Errors, Warnings, deprecation_warning
from .. import util
from .underscore import Underscore, get_ext_args
from ._retokenize import Retokenizer

DEF PADDING = 5


cdef int bounds_check(int i, int length, int padding) except -1:
    if (i + padding) < 0:
        raise IndexError(Errors.E026.format(i=i, length=length))
    if (i - padding) >= length:
        raise IndexError(Errors.E026.format(i=i, length=length))


cdef attr_t get_token_attr(const TokenC* token, attr_id_t feat_name) nogil:
    if feat_name == LEMMA:
        return token.lemma
    elif feat_name == POS:
        return token.pos
    elif feat_name == TAG:
        return token.tag
    elif feat_name == DEP:
        return token.dep
    elif feat_name == HEAD:
        return token.head
    elif feat_name == SENT_START:
        return token.sent_start
    elif feat_name == SPACY:
        return token.spacy
    elif feat_name == ENT_IOB:
        return token.ent_iob
    elif feat_name == ENT_TYPE:
        return token.ent_type
    else:
        return Lexeme.get_struct_attr(token.lex, feat_name)


def _get_chunker(lang):
    try:
        cls = util.get_lang_class(lang)
    except ImportError:
        return None
    except KeyError:
        return None
    return cls.Defaults.syntax_iterators.get(u'noun_chunks')


cdef class Doc:
    """A sequence of Token objects. Access sentences and named entities, export
    annotations to numpy arrays, losslessly serialize to compressed binary
    strings. The `Doc` object holds an array of `TokenC` structs. The
    Python-level `Token` and `Span` objects are views of this array, i.e.
    they don't own the data themselves.

    EXAMPLE: Construction 1
        >>> doc = nlp(u'Some text')

        Construction 2
        >>> from spacy.tokens import Doc
        >>> doc = Doc(nlp.vocab, words=[u'hello', u'world', u'!'],
                      spaces=[True, False, False])
    """
    @classmethod
    def set_extension(cls, name, **kwargs):
        if cls.has_extension(name) and not kwargs.get('force', False):
            raise ValueError(Errors.E090.format(name=name, obj='Doc'))
        Underscore.doc_extensions[name] = get_ext_args(**kwargs)

    @classmethod
    def get_extension(cls, name):
        return Underscore.doc_extensions.get(name)

    @classmethod
    def has_extension(cls, name):
        return name in Underscore.doc_extensions

    @classmethod
    def remove_extension(cls, name):
        if not cls.has_extension(name):
            raise ValueError(Errors.E046.format(name=name))
        return Underscore.doc_extensions.pop(name)

    def __init__(self, Vocab vocab, words=None, spaces=None, user_data=None,
                 orths_and_spaces=None):
        """Create a Doc object.

        vocab (Vocab): A vocabulary object, which must match any models you
            want to use (e.g. tokenizer, parser, entity recognizer).
        words (list or None): A list of unicode strings to add to the document
            as words. If `None`, defaults to empty list.
        spaces (list or None): A list of boolean values, of the same length as
            words. True means that the word is followed by a space, False means
            it is not. If `None`, defaults to `[True]*len(words)`
        user_data (dict or None): Optional extra data to attach to the Doc.
        RETURNS (Doc): The newly constructed object.
        """
        self.vocab = vocab
        size = 20
        self.mem = Pool()
        # Guarantee self.lex[i-x], for any i >= 0 and x < padding is in bounds
        # However, we need to remember the true starting places, so that we can
        # realloc.
        data_start = <TokenC*>self.mem.alloc(size + (PADDING*2), sizeof(TokenC))
        cdef int i
        for i in range(size + (PADDING*2)):
            data_start[i].lex = &EMPTY_LEXEME
            data_start[i].l_edge = i
            data_start[i].r_edge = i
        self.c = data_start + PADDING
        self.max_length = size
        self.length = 0
        self.is_tagged = False
        self.is_parsed = False
        self.sentiment = 0.0
        self.cats = {}
        self.user_hooks = {}
        self.user_token_hooks = {}
        self.user_span_hooks = {}
        self.tensor = numpy.zeros((0,), dtype='float32')
        self.user_data = {} if user_data is None else user_data
        self._vector = None
        self.noun_chunks_iterator = _get_chunker(self.vocab.lang)
        cdef unicode orth
        cdef bint has_space
        if orths_and_spaces is None and words is not None:
            if spaces is None:
                spaces = [True] * len(words)
            elif len(spaces) != len(words):
                raise ValueError(Errors.E027)
            orths_and_spaces = zip(words, spaces)
        if orths_and_spaces is not None:
            for orth_space in orths_and_spaces:
                if isinstance(orth_space, unicode):
                    orth = orth_space
                    has_space = True
                elif isinstance(orth_space, bytes):
                    raise ValueError(Errors.E028.format(value=orth_space))
                else:
                    orth, has_space = orth_space
                # Note that we pass self.mem here --- we have ownership, if LexemeC
                # must be created.
                self.push_back(
                    <const LexemeC*>self.vocab.get(self.mem, orth), has_space)
        # Tough to decide on policy for this. Is an empty doc tagged and parsed?
        # There's no information we'd like to add to it, so I guess so?
        if self.length == 0:
            self.is_tagged = True
            self.is_parsed = True

    @property
    def _(self):
        return Underscore(Underscore.doc_extensions, self)

    @property
    def is_sentenced(self):
        # Check if the document has sentence boundaries,
        # i.e at least one tok has the sent_start in (-1, 1)
        if 'sents' in self.user_hooks:
            return True
        if self.is_parsed:
            return True
        for i in range(self.length):
            if self.c[i].sent_start == -1 or self.c[i].sent_start == 1:
                return True
        else:
            return False

    def __getitem__(self, object i):
        """Get a `Token` or `Span` object.

        i (int or tuple) The index of the token, or the slice of the document
            to get.
        RETURNS (Token or Span): The token at `doc[i]]`, or the span at
            `doc[start : end]`.

        EXAMPLE:
            >>> doc[i]
            Get the `Token` object at position `i`, where `i` is an integer.
            Negative indexing is supported, and follows the usual Python
            semantics, i.e. `doc[-2]` is `doc[len(doc) - 2]`.

            >>> doc[start : end]]
            Get a `Span` object, starting at position `start` and ending at
            position `end`, where `start` and `end` are token indices. For
            instance, `doc[2:5]` produces a span consisting of tokens 2, 3 and
            4. Stepped slices (e.g. `doc[start : end : step]`) are not
            supported, as `Span` objects must be contiguous (cannot have gaps).
            You can use negative indices and open-ended ranges, which have
            their normal Python semantics.
        """
        if isinstance(i, slice):
            start, stop = normalize_slice(len(self), i.start, i.stop, i.step)
            return Span(self, start, stop, label=0)

        if i < 0:
            i = self.length + i
        bounds_check(i, self.length, PADDING)
        return Token.cinit(self.vocab, &self.c[i], i, self)

    def __iter__(self):
        """Iterate over `Token`  objects, from which the annotations can be
        easily accessed. This is the main way of accessing `Token` objects,
        which are the main way annotations are accessed from Python. If faster-
        than-Python speeds are required, you can instead access the annotations
        as a numpy array, or access the underlying C data directly from Cython.

        EXAMPLE:
            >>> for token in doc
        """
        cdef int i
        for i in range(self.length):
            yield Token.cinit(self.vocab, &self.c[i], i, self)

    def __len__(self):
        """The number of tokens in the document.

        RETURNS (int): The number of tokens in the document.

        EXAMPLE:
            >>> len(doc)
        """
        return self.length

    def __unicode__(self):
        return u''.join([t.text_with_ws for t in self])

    def __bytes__(self):
        return u''.join([t.text_with_ws for t in self]).encode('utf-8')

    def __str__(self):
        if is_config(python3=True):
            return self.__unicode__()
        return self.__bytes__()

    def __repr__(self):
        return self.__str__()

    @property
    def doc(self):
        return self

    def char_span(self, int start_idx, int end_idx, label=0, vector=None):
        """Create a `Span` object from the slice `doc.text[start : end]`.

        doc (Doc): The parent document.
        start (int): The index of the first character of the span.
        end (int): The index of the first character after the span.
        label (uint64 or string): A label to attach to the Span, e.g. for
            named entities.
        vector (ndarray[ndim=1, dtype='float32']): A meaning representation of
            the span.
        RETURNS (Span): The newly constructed object.
        """
        if not isinstance(label, int):
            label = self.vocab.strings.add(label)
        cdef int start = token_by_start(self.c, self.length, start_idx)
        if start == -1:
            return None
        cdef int end = token_by_end(self.c, self.length, end_idx)
        if end == -1:
            return None
        # Currently we have the token index, we want the range-end index
        end += 1
        cdef Span span = Span(self, start, end, label=label, vector=vector)
        return span

    def similarity(self, other):
        """Make a semantic similarity estimate. The default estimate is cosine
        similarity using an average of word vectors.

        other (object): The object to compare with. By default, accepts `Doc`,
            `Span`, `Token` and `Lexeme` objects.
        RETURNS (float): A scalar similarity score. Higher is more similar.
        """
        if 'similarity' in self.user_hooks:
            return self.user_hooks['similarity'](self, other)
        if isinstance(other, (Lexeme, Token)) and self.length == 1:
            if self.c[0].lex.orth == other.orth:
                return 1.0
        elif isinstance(other, (Span, Doc)):
            if len(self) == len(other):
                for i in range(self.length):
                    if self[i].orth != other[i].orth:
                        break
                else:
                    return 1.0

        if self.vector_norm == 0 or other.vector_norm == 0:
            return 0.0
        return numpy.dot(self.vector, other.vector) / (self.vector_norm * other.vector_norm)

    property has_vector:
        """A boolean value indicating whether a word vector is associated with
        the object.

        RETURNS (bool): Whether a word vector is associated with the object.
        """
        def __get__(self):
            if 'has_vector' in self.user_hooks:
                return self.user_hooks['has_vector'](self)
            elif self.vocab.vectors.data.size:
                return True
            elif self.tensor.size:
                return True
            else:
                return False

    property vector:
        """A real-valued meaning representation. Defaults to an average of the
        token vectors.

        RETURNS (numpy.ndarray[ndim=1, dtype='float32']): A 1D numpy array
            representing the document's semantics.
        """
        def __get__(self):
            if 'vector' in self.user_hooks:
                return self.user_hooks['vector'](self)
            if self._vector is not None:
                return self._vector
            elif not len(self):
                self._vector = numpy.zeros((self.vocab.vectors_length,),
                                           dtype='f')
                return self._vector
            elif self.vocab.vectors.data.size > 0:
                vector = numpy.zeros((self.vocab.vectors_length,), dtype='f')
                for token in self.c[:self.length]:
                    vector += self.vocab.get_vector(token.lex.orth)
                self._vector = vector / len(self)
                return self._vector
            elif self.tensor.size > 0:
                self._vector = self.tensor.mean(axis=0)
                return self._vector
            else:
                return numpy.zeros((self.vocab.vectors_length,),
                                   dtype='float32')

        def __set__(self, value):
            self._vector = value

    property vector_norm:
        """The L2 norm of the document's vector representation.

        RETURNS (float): The L2 norm of the vector representation.
        """
        def __get__(self):
            if 'vector_norm' in self.user_hooks:
                return self.user_hooks['vector_norm'](self)
            cdef float value
            cdef double norm = 0
            if self._vector_norm is None:
                norm = 0.0
                for value in self.vector:
                    norm += value * value
                self._vector_norm = sqrt(norm) if norm != 0 else 0
            return self._vector_norm

        def __set__(self, value):
            self._vector_norm = value

    property text:
        """A unicode representation of the document text.

        RETURNS (unicode): The original verbatim text of the document.
        """
        def __get__(self):
            return u''.join(t.text_with_ws for t in self)

    property text_with_ws:
        """An alias of `Doc.text`, provided for duck-type compatibility with
        `Span` and `Token`.

        RETURNS (unicode): The original verbatim text of the document.
        """
        def __get__(self):
            return self.text

    property ents:
        """Iterate over the entities in the document. Yields named-entity
        `Span` objects, if the entity recognizer has been applied to the
        document.

        YIELDS (Span): Entities in the document.

        EXAMPLE: Iterate over the span to get individual Token objects,
            or access the label:

            >>> tokens = nlp(u'Mr. Best flew to New York on Saturday morning.')
            >>> ents = list(tokens.ents)
            >>> assert ents[0].label == 346
            >>> assert ents[0].label_ == 'PERSON'
            >>> assert ents[0].orth_ == 'Best'
            >>> assert ents[0].text == 'Mr. Best'
        """
        def __get__(self):
            cdef int i
            cdef const TokenC* token
            cdef int start = -1
            cdef attr_t label = 0
            output = []
            for i in range(self.length):
                token = &self.c[i]
                if token.ent_iob == 1:
                    if start == -1:
                        seq = ['%s|%s' % (t.text, t.ent_iob_) for t in self[i-5:i+5]]
                        raise ValueError(Errors.E093.format(seq=' '.join(seq)))
                elif token.ent_iob == 2 or token.ent_iob == 0:
                    if start != -1:
                        output.append(Span(self, start, i, label=label))
                    start = -1
                    label = 0
                elif token.ent_iob == 3:
                    if start != -1:
                        output.append(Span(self, start, i, label=label))
                    start = i
                    label = token.ent_type
            if start != -1:
                output.append(Span(self, start, self.length, label=label))
            return tuple(output)

        def __set__(self, ents):
            # TODO:
            # 1. Allow negative matches
            # 2. Ensure pre-set NERs are not over-written during statistical
            #    prediction
            # 3. Test basic data-driven ORTH gazetteer
            # 4. Test more nuanced date and currency regex
            cdef int i
            for i in range(self.length):
                self.c[i].ent_type = 0
                self.c[i].ent_iob = 0  # Means missing.
            cdef attr_t ent_type
            cdef int start, end
            for ent_info in ents:
                if isinstance(ent_info, Span):
                    ent_id = ent_info.ent_id
                    ent_type = ent_info.label
                    start = ent_info.start
                    end = ent_info.end
                elif len(ent_info) == 3:
                    ent_type, start, end = ent_info
                else:
                    ent_id, ent_type, start, end = ent_info
                if ent_type is None or ent_type < 0:
                    # Mark as O
                    for i in range(start, end):
                        self.c[i].ent_type = 0
                        self.c[i].ent_iob = 2
                else:
                    # Mark (inside) as I
                    for i in range(start, end):
                        self.c[i].ent_type = ent_type
                        self.c[i].ent_iob = 1
                    # Set start as B
                    self.c[start].ent_iob = 3

    property noun_chunks:
        """Iterate over the base noun phrases in the document. Yields base
        noun-phrase #[code Span] objects, if the document has been
        syntactically parsed. A base noun phrase, or "NP chunk", is a noun
        phrase that does not permit other NPs to be nested within it – so no
        NP-level coordination, no prepositional phrases, and no relative
        clauses.

        YIELDS (Span): Noun chunks in the document.
        """
        def __get__(self):
            if not self.is_parsed:
                raise ValueError(Errors.E029)
            # Accumulate the result before beginning to iterate over it. This
            # prevents the tokenisation from being changed out from under us
            # during the iteration. The tricky thing here is that Span accepts
            # its tokenisation changing, so it's okay once we have the Span
            # objects. See Issue #375.
            spans = []
            if self.noun_chunks_iterator is not None:
                for start, end, label in self.noun_chunks_iterator(self):
                    spans.append(Span(self, start, end, label=label))
            for span in spans:
                yield span

    property sents:
        """Iterate over the sentences in the document. Yields sentence `Span`
        objects. Sentence spans have no label. To improve accuracy on informal
        texts, spaCy calculates sentence boundaries from the syntactic
        dependency parse. If the parser is disabled, the `sents` iterator will
        be unavailable.

        EXAMPLE:
            >>> doc = nlp("This is a sentence. Here's another...")
            >>> assert [s.root.text for s in doc.sents] == ["is", "'s"]
        """
        def __get__(self):
            if not self.is_sentenced:
                raise ValueError(Errors.E030)
            if 'sents' in self.user_hooks:
                yield from self.user_hooks['sents'](self)
            else:
                start = 0
                for i in range(1, self.length):
                    if self.c[i].sent_start == 1:
                        yield Span(self, start, i)
                        start = i
                if start != self.length:
                    yield Span(self, start, self.length)

    cdef int push_back(self, LexemeOrToken lex_or_tok, bint has_space) except -1:
        if self.length == 0:
            # Flip these to false when we see the first token.
            self.is_tagged = False
            self.is_parsed = False
        if self.length == self.max_length:
            self._realloc(self.length * 2)
        cdef TokenC* t = &self.c[self.length]
        if LexemeOrToken is const_TokenC_ptr:
            t[0] = lex_or_tok[0]
        else:
            t.lex = lex_or_tok
        if self.length == 0:
            t.idx = 0
        else:
            t.idx = (t-1).idx + (t-1).lex.length + (t-1).spacy
        t.l_edge = self.length
        t.r_edge = self.length
        if t.lex.orth == 0:
            raise ValueError(Errors.E031.format(i=self.length))
        t.spacy = has_space
        self.length += 1
        return t.idx + t.lex.length + t.spacy

    @cython.boundscheck(False)
    cpdef np.ndarray to_array(self, object py_attr_ids):
        """Export given token attributes to a numpy `ndarray`.
        If `attr_ids` is a sequence of M attributes, the output array will be
        of shape `(N, M)`, where N is the length of the `Doc` (in tokens). If
        `attr_ids` is a single attribute, the output shape will be (N,). You
        can specify attributes by integer ID (e.g. spacy.attrs.LEMMA) or
        string name (e.g. 'LEMMA' or 'lemma').

        attr_ids (list[]): A list of attributes (int IDs or string names).
        RETURNS (numpy.ndarray[long, ndim=2]): A feature matrix, with one row
            per word, and one column per attribute indicated in the input
            `attr_ids`.

        EXAMPLE:
            >>> from spacy.attrs import LOWER, POS, ENT_TYPE, IS_ALPHA
            >>> doc = nlp(text)
            >>> # All strings mapped to integers, for easy export to numpy
            >>> np_array = doc.to_array([LOWER, POS, ENT_TYPE, IS_ALPHA])
        """
        cdef int i, j
        cdef attr_id_t feature
        cdef np.ndarray[attr_t, ndim=2] output
        # Handle scalar/list inputs of strings/ints for py_attr_ids
        if not hasattr(py_attr_ids, '__iter__') \
        and not isinstance(py_attr_ids, basestring_):
            py_attr_ids = [py_attr_ids]

        # Allow strings, e.g. 'lemma' or 'LEMMA'
        py_attr_ids = [(IDS[id_.upper()] if hasattr(id_, 'upper') else id_)
                       for id_ in py_attr_ids]
        # Make an array from the attributes --- otherwise our inner loop is
        # Python dict iteration.
        cdef np.ndarray attr_ids = numpy.asarray(py_attr_ids, dtype='i')
        output = numpy.ndarray(shape=(self.length, len(attr_ids)),
                               dtype=numpy.uint64)
        c_output = <attr_t*>output.data
        c_attr_ids = <attr_id_t*>attr_ids.data
        cdef TokenC* token
        cdef int nr_attr = attr_ids.shape[0]
        for i in range(self.length):
            token = &self.c[i]
            for j in range(nr_attr):
                c_output[i*nr_attr + j] = get_token_attr(token, c_attr_ids[j])
        # Handle 1d case
        return output if len(attr_ids) >= 2 else output.reshape((self.length,))

    def count_by(self, attr_id_t attr_id, exclude=None,
                 PreshCounter counts=None):
        """Count the frequencies of a given attribute. Produces a dict of
        `{attribute (int): count (ints)}` frequencies, keyed by the values of
        the given attribute ID.

        attr_id (int): The attribute ID to key the counts.
        RETURNS (dict): A dictionary mapping attributes to integer counts.

        EXAMPLE:
            >>> from spacy import attrs
            >>> doc = nlp(u'apple apple orange banana')
            >>> tokens.count_by(attrs.ORTH)
            {12800L: 1, 11880L: 2, 7561L: 1}
            >>> tokens.to_array([attrs.ORTH])
            array([[11880], [11880], [7561], [12800]])
        """
        cdef int i
        cdef attr_t attr
        cdef size_t count

        if counts is None:
            counts = PreshCounter()
            output_dict = True
        else:
            output_dict = False
        # Take this check out of the loop, for a bit of extra speed
        if exclude is None:
            for i in range(self.length):
                counts.inc(get_token_attr(&self.c[i], attr_id), 1)
        else:
            for i in range(self.length):
                if not exclude(self[i]):
                    attr = get_token_attr(&self.c[i], attr_id)
                    counts.inc(attr, 1)
        if output_dict:
            return dict(counts)

    def _realloc(self, new_size):
        self.max_length = new_size
        n = new_size + (PADDING * 2)
        # What we're storing is a "padded" array. We've jumped forward PADDING
        # places, and are storing the pointer to that. This way, we can access
        # words out-of-bounds, and get out-of-bounds markers.
        # Now that we want to realloc, we need the address of the true start,
        # so we jump the pointer back PADDING places.
        cdef TokenC* data_start = self.c - PADDING
        data_start = <TokenC*>self.mem.realloc(data_start, n * sizeof(TokenC))
        self.c = data_start + PADDING
        cdef int i
        for i in range(self.length, self.max_length + PADDING):
            self.c[i].lex = &EMPTY_LEXEME

    cdef void set_parse(self, const TokenC* parsed) nogil:
        # TODO: This method is fairly misleading atm. It's used by Parser
        # to actually apply the parse calculated. Need to rethink this.

        # Probably we should use from_array?
        self.is_parsed = True
        for i in range(self.length):
            self.c[i] = parsed[i]

    def from_array(self, attrs, array):
        if SENT_START in attrs and HEAD in attrs:
            raise ValueError(Errors.E032)
        cdef int i, col
        cdef attr_id_t attr_id
        cdef TokenC* tokens = self.c
        cdef int length = len(array)
        # Get set up for fast loading
        cdef Pool mem = Pool()
        cdef int n_attrs = len(attrs)
        attr_ids = <attr_id_t*>mem.alloc(n_attrs, sizeof(attr_id_t))
        for i, attr_id in enumerate(attrs):
            attr_ids[i] = attr_id
        # Now load the data
        for i in range(self.length):
            token = &self.c[i]
            for j in range(n_attrs):
                Token.set_struct_attr(token, attr_ids[j], array[i, j])
        # Auxiliary loading logic
        for col, attr_id in enumerate(attrs):
            if attr_id == TAG:
                for i in range(length):
                    if array[i, col] != 0:
                        self.vocab.morphology.assign_tag(&tokens[i], array[i, col])
        # set flags
        self.is_parsed = bool(HEAD in attrs or DEP in attrs)
        self.is_tagged = bool(TAG in attrs or POS in attrs)
        # if document is parsed, set children
        if self.is_parsed:
            set_children_from_heads(self.c, self.length)
        return self

    def get_lca_matrix(self):
        """Calculates the lowest common ancestor matrix for a given `Doc`.
        Returns LCA matrix containing the integer index of the ancestor, or -1
        if no common ancestor is found (ex if span excludes a necessary
        ancestor). Apologies about the recursion, but the impact on
        performance is negligible given the natural limitations on the depth
        of a typical human sentence.
        """
        # Efficiency notes:
        # We can easily improve the performance here by iterating in Cython.
        # To loop over the tokens in Cython, the easiest way is:
        # for token in doc.c[:doc.c.length]:
        #     head = token + token.head
        # Both token and head will be TokenC* here. The token.head attribute
        # is an integer offset.
        def __pairwise_lca(token_j, token_k, lca_matrix):
            if lca_matrix[token_j.i][token_k.i] != -2:
                return lca_matrix[token_j.i][token_k.i]
            elif token_j == token_k:
                lca_index = token_j.i
            elif token_k.head == token_j:
                lca_index = token_j.i
            elif token_j.head == token_k:
                lca_index = token_k.i
            elif (token_j.head == token_j) and (token_k.head == token_k):
                lca_index = -1
            else:
                lca_index = __pairwise_lca(token_j.head, token_k.head,
                                           lca_matrix)
            lca_matrix[token_j.i][token_k.i] = lca_index
            lca_matrix[token_k.i][token_j.i] = lca_index

            return lca_index

        lca_matrix = numpy.empty((len(self), len(self)), dtype=numpy.int32)
        lca_matrix.fill(-2)
        for j in range(len(self)):
            token_j = self[j]
            for k in range(j, len(self)):
                token_k = self[k]
                lca_matrix[j][k] = __pairwise_lca(token_j, token_k, lca_matrix)
                lca_matrix[k][j] = lca_matrix[j][k]
        return lca_matrix

    def to_disk(self, path, **exclude):
        """Save the current state to a directory.

        path (unicode or Path): A path to a directory, which will be created if
            it doesn't exist. Paths may be either strings or Path-like objects.
        """
        path = util.ensure_path(path)
        with path.open('wb') as file_:
            file_.write(self.to_bytes(**exclude))

    def from_disk(self, path, **exclude):
        """Loads state from a directory. Modifies the object in place and
        returns it.

        path (unicode or Path): A path to a directory. Paths may be either
            strings or `Path`-like objects.
        RETURNS (Doc): The modified `Doc` object.
        """
        path = util.ensure_path(path)
        with path.open('rb') as file_:
            bytes_data = file_.read()
        return self.from_bytes(bytes_data, **exclude)

    def to_bytes(self, **exclude):
        """Serialize, i.e. export the document contents to a binary string.

        RETURNS (bytes): A losslessly serialized copy of the `Doc`, including
            all annotations.
        """
        array_head = [LENGTH, SPACY, LEMMA, ENT_IOB, ENT_TYPE]

        if self.is_tagged:
            array_head.append(TAG)
        # if doc parsed add head and dep attribute
        if self.is_parsed:
            array_head.extend([HEAD, DEP])
        # otherwise add sent_start
        else:
            array_head.append(SENT_START)
        # Msgpack doesn't distinguish between lists and tuples, which is
        # vexing for user data. As a best guess, we *know* that within
        # keys, we must have tuples. In values we just have to hope
        # users don't mind getting a list instead of a tuple.
        serializers = {
            'text': lambda: self.text,
            'array_head': lambda: array_head,
            'array_body': lambda: self.to_array(array_head),
            'sentiment': lambda: self.sentiment,
            'tensor': lambda: self.tensor,
        }
        if 'user_data' not in exclude and self.user_data:
            user_data_keys, user_data_values = list(zip(*self.user_data.items()))
            serializers['user_data_keys'] = lambda: msgpack.dumps(user_data_keys)
            serializers['user_data_values'] = lambda: msgpack.dumps(user_data_values)

        return util.to_bytes(serializers, exclude)

    def from_bytes(self, bytes_data, **exclude):
        """Deserialize, i.e. import the document contents from a binary string.

        data (bytes): The string to load from.
        RETURNS (Doc): Itself.
        """
        if self.length != 0:
            raise ValueError(Errors.E033.format(length=self.length))
        deserializers = {
            'text': lambda b: None,
            'array_head': lambda b: None,
            'array_body': lambda b: None,
            'sentiment': lambda b: None,
            'tensor': lambda b: None,
            'user_data_keys': lambda b: None,
            'user_data_values': lambda b: None,
        }

        msg = util.from_bytes(bytes_data, deserializers, exclude)
        # Msgpack doesn't distinguish between lists and tuples, which is
        # vexing for user data. As a best guess, we *know* that within
        # keys, we must have tuples. In values we just have to hope
        # users don't mind getting a list instead of a tuple.
        if 'user_data' not in exclude and 'user_data_keys' in msg:
            user_data_keys = msgpack.loads(msg['user_data_keys'],
                                           use_list=False, raw=False)
            user_data_values = msgpack.loads(msg['user_data_values'], raw=False)
            for key, value in zip(user_data_keys, user_data_values):
                self.user_data[key] = value

        cdef int i, start, end, has_space

        if 'sentiment' not in exclude and 'sentiment' in msg:
            self.sentiment = msg['sentiment']
        if 'tensor' not in exclude and 'tensor' in msg:
            self.tensor = msg['tensor']

        start = 0
        cdef const LexemeC* lex
        cdef unicode orth_
        text = msg['text']
        attrs = msg['array_body']
        for i in range(attrs.shape[0]):
            end = start + attrs[i, 0]
            has_space = attrs[i, 1]
            orth_ = text[start:end]
            lex = self.vocab.get(self.mem, orth_)
            self.push_back(lex, has_space)
            start = end + has_space
        self.from_array(msg['array_head'][2:], attrs[:, 2:])
        return self

    def extend_tensor(self, tensor):
        '''Concatenate a new tensor onto the doc.tensor object.

        The doc.tensor attribute holds dense feature vectors
        computed by the models in the pipeline. Let's say a
        document with 30 words has a tensor with 128 dimensions
        per word. doc.tensor.shape will be (30, 128). After
        calling doc.extend_tensor with an array of shape (30, 64),
        doc.tensor == (30, 192).
        '''
        xp = get_array_module(self.tensor)
        if self.tensor.size == 0:
            self.tensor.resize(tensor.shape)
            copy_array(self.tensor, tensor)
        else:
            self.tensor = xp.hstack((self.tensor, tensor))

    def retokenize(self):
        '''Context manager to handle retokenization of the Doc.
        Modifications to the Doc's tokenization are stored, and then
        made all at once when the context manager exits. This is
        much more efficient, and less error-prone.

        All views of the Doc (Span and Token) created before the
        retokenization are invalidated, although they may accidentally
        continue to work.
        '''
        return Retokenizer(self)

    def merge(self, int start_idx, int end_idx, *args, **attributes):
        """Retokenize the document, such that the span at
        `doc.text[start_idx : end_idx]` is merged into a single token. If
        `start_idx` and `end_idx `do not mark start and end token boundaries,
        the document remains unchanged.

        start_idx (int): Character index of the start of the slice to merge.
        end_idx (int): Character index after the end of the slice to merge.
        **attributes: Attributes to assign to the merged token. By default,
            attributes are inherited from the syntactic root of the span.
        RETURNS (Token): The newly merged token, or `None` if the start and end
            indices did not fall at token boundaries.
        """
        cdef unicode tag, lemma, ent_type
        if len(args) == 3:
            deprecation_warning(Warnings.W003)
            tag, lemma, ent_type = args
            attributes[TAG] = tag
            attributes[LEMMA] = lemma
            attributes[ENT_TYPE] = ent_type
        elif not args:
            if 'label' in attributes and 'ent_type' not in attributes:
                if isinstance(attributes['label'], int):
                    attributes[ENT_TYPE] = attributes['label']
                else:
                    attributes[ENT_TYPE] = self.vocab.strings[attributes['label']]
            if 'ent_type' in attributes:
                attributes[ENT_TYPE] = attributes['ent_type']
        elif args:
            raise ValueError(Errors.E034.format(n_args=len(args),
                                                args=repr(args),
                                                kwargs=repr(attributes)))
        # More deprecated attribute handling =/
        if 'label' in attributes:
            attributes['ent_type'] = attributes.pop('label')

        attributes = intify_attrs(attributes, strings_map=self.vocab.strings)

        cdef int start = token_by_start(self.c, self.length, start_idx)
        if start == -1:
            return None
        cdef int end = token_by_end(self.c, self.length, end_idx)
        if end == -1:
            return None
        # Currently we have the token index, we want the range-end index
        end += 1
        with self.retokenize() as retokenizer:
            retokenizer.merge(self[start:end], attrs=attributes)
        return self[start]

    def print_tree(self, light=False, flat=False):
        """Returns the parse trees in JSON (dict) format.

        light (bool): Don't include lemmas or entities.
        flat (bool): Don't include arcs or modifiers.
        RETURNS (dict): Parse tree as dict.

        EXAMPLE:
            >>> doc = nlp('Bob brought Alice the pizza. Alice ate the pizza.')
            >>> trees = doc.print_tree()
            >>> trees[1]
            {'modifiers': [
                {'modifiers': [], 'NE': 'PERSON', 'word': 'Alice',
                'arc': 'nsubj', 'POS_coarse': 'PROPN', 'POS_fine': 'NNP',
                'lemma': 'Alice'},
                {'modifiers': [
                    {'modifiers': [], 'NE': '', 'word': 'the', 'arc': 'det',
                    'POS_coarse': 'DET', 'POS_fine': 'DT', 'lemma': 'the'}],
                'NE': '', 'word': 'pizza', 'arc': 'dobj', 'POS_coarse': 'NOUN',
                'POS_fine': 'NN', 'lemma': 'pizza'},
                {'modifiers': [], 'NE': '', 'word': '.', 'arc': 'punct',
                'POS_coarse': 'PUNCT', 'POS_fine': '.', 'lemma': '.'}],
                'NE': '', 'word': 'ate', 'arc': 'ROOT', 'POS_coarse': 'VERB',
                'POS_fine': 'VBD', 'lemma': 'eat'}
        """
        return parse_tree(self, light=light, flat=flat)


cdef int token_by_start(const TokenC* tokens, int length, int start_char) except -2:
    cdef int i
    for i in range(length):
        if tokens[i].idx == start_char:
            return i
    else:
        return -1


cdef int token_by_end(const TokenC* tokens, int length, int end_char) except -2:
    cdef int i
    for i in range(length):
        if tokens[i].idx + tokens[i].lex.length == end_char:
            return i
    else:
        return -1


cdef int set_children_from_heads(TokenC* tokens, int length) except -1:
    cdef TokenC* head
    cdef TokenC* child
    cdef int i
    # Set number of left/right children to 0. We'll increment it in the loops.
    for i in range(length):
        tokens[i].l_kids = 0
        tokens[i].r_kids = 0
        tokens[i].l_edge = i
        tokens[i].r_edge = i
    # Set left edges
    for i in range(length):
        child = &tokens[i]
        head = &tokens[i + child.head]
        if child < head:
            head.l_kids += 1
        if child.l_edge < head.l_edge:
            head.l_edge = child.l_edge

    # Set right edges --- same as above, but iterate in reverse
    for i in range(length-1, -1, -1):
        child = &tokens[i]
        head = &tokens[i + child.head]
        if child > head:
            head.r_kids += 1
        if child.r_edge > head.r_edge:
            head.r_edge = child.r_edge


    # Set sentence starts
    for i in range(length):
        if tokens[i].head == 0 and tokens[i].dep != 0:
            tokens[tokens[i].l_edge].sent_start = True


def pickle_doc(doc):
    bytes_data = doc.to_bytes(vocab=False, user_data=False)
    hooks_and_data = (doc.user_data, doc.user_hooks, doc.user_span_hooks,
                      doc.user_token_hooks)
    return (unpickle_doc, (doc.vocab, dill.dumps(hooks_and_data), bytes_data))


def unpickle_doc(vocab, hooks_and_data, bytes_data):
    user_data, doc_hooks, span_hooks, token_hooks = dill.loads(hooks_and_data)

    doc = Doc(vocab, user_data=user_data).from_bytes(bytes_data,
                                                     exclude='user_data')
    doc.user_hooks.update(doc_hooks)
    doc.user_span_hooks.update(span_hooks)
    doc.user_token_hooks.update(token_hooks)
    return doc


copy_reg.pickle(Doc, pickle_doc, unpickle_doc)



'''Test that models with no pretrained vectors can be deserialized correctly
after vectors are added.'''
from __future__ import unicode_literals
import numpy
from ...pipeline import Tagger
from ...vectors import Vectors
from ...vocab import Vocab
from ..util import make_tempdir


def test_issue1727():
    data = numpy.ones((3, 300), dtype='f')
    keys = [u'I', u'am', u'Matt']
    vectors = Vectors(data=data, keys=keys)
    tagger = Tagger(Vocab())
    tagger.add_label('PRP')
    tagger.begin_training()

    assert tagger.cfg.get('pretrained_dims', 0) == 0
    tagger.vocab.vectors = vectors

    with make_tempdir() as path:
        tagger.to_disk(path)
        tagger = Tagger(Vocab()).from_disk(path)
        assert tagger.cfg.get('pretrained_dims', 0) == 0



# coding: utf-8
from __future__ import unicode_literals

import pytest

@pytest.mark.parametrize('string,lemma', [('evlerimizdeki', 'ev'),
                                          ('işlerimizi', 'iş'),
                                          ('biran', 'biran'),
                                          ('bitirmeliyiz', 'bitir'),
                                          ('isteklerimizi', 'istek'),
                                          ('karşılaştırmamızın', 'karşılaştır'),
                                          ('çoğulculuktan', 'çoğulcu')])
def test_lemmatizer_lookup_assigns(tr_tokenizer, string, lemma):
    tokens = tr_tokenizer(string)
    assert tokens[0].lemma_ == lemma


from __future__ import unicode_literals
PROPER_NAMES = set("""
 άαχεν άβαρος άβδηρα άβελ άβιλα άβολα άγγελοι άγγελος άγιο πνεύμα
 άγιοι τόποι άγιον όρος άγιος αθανάσιος άγιος αναστάσιος άγιος αντώνιος
 άγιος αριστείδης άγιος βαρθολομαίος άγιος βασίλειος άγιος βασίλης
 άγιος γεράσιμος άγιος γεώργιος άγιος γρηγόριος άγιος δημήτριος άγιος διονύσιος
 άγιος δομίνικος άγιος ελευθέριος άγιος εφραίμ άγιος θαλέλαιος άγιος θεόδωρος
 άγιος ιωάννης άγιος κωνσταντίνος άγιος λεωνάρδος άγιος λεωνίδας άγιος λουκάς
 άγιος μάρκος άγιος μήνας άγιος μαρίνος άγιος μαρτίνος άγιος μιλτιάδης
 άγιος νικηφόρος άγιος νικόδημος άγιος νικόλαος άγιος παναγιώτης
 άγιος παύλος άγιος ροβέρτος άγιος σεβαστιανός άγιος στέφανος άγιος τωβίας
 άγιος φλώρος άγιος φώτιος άγιος χριστόδουλος άγιος χριστόφορος
 άγιος χρυσόστομος άγις άγκυρα άγραφα άδανα άδης άδμητος άδρα άδωνης άδωνις
 άιφελ άλκαμο άλκης άλκηστις άλκμααρ άλμπα ιούλια άλπεις άλσφελτ άλτα βεραπάς
 άμστερνταμ άμφισσα άμων άνδεις άνδρος άνθεια άνθιμος άννα άννυ άντα άντζελα
 άντρια άντσιο άνω απρόβατο άνω νευροκόπι άνω ποταμιά άξονας άουγκσμπουργκ
 άραξης άρβα άργος άρδανα άρειος πάγος άρης άριος άρκτος άρνεμ άρνη άρτα άρτεμη
 άσσος άστι άστρος άτλας άτροπος άτταλος έβελιν έβερεστ έβορα έβρος έδεσσα
 έκτωρ έλενα έλενος έλλη έλλην έλληνας έλτσε έμιλυ έμιλυ ντίκινσον έμμα έμπολα
 έμυ ένδοξοι νήσοι έξαρχος έπαφος έρασμος έριδα έρις έρκυνα έρμπα έρση έρως
 έσλιγκεν έσπερος έσσεν έστε έφεσος έφη έχιδνα ήβη ήλιος ήπειρος ήρα ήρων
 ήφαιστος ίαιρος ίακχος ίαμος ίασμος ίβυκος ίκαρος ίκι ίμβρος ίμια ίμολα ίναχος
 ίνσμπρουκ ίος ίουλος ίππαρχος ίριδα ίρις ίσα ίστρος ίων ίωνας αΐρα αέρες
 αίας αίγινα αίγιο αίγισθος αίγλη αίγλων αίγυπτος αίθρα αίμος αίμων αίολος
 αίτνα ααρών αβάνα αβέιρο αβέρκιος αβίς αβαλόν αβαροσλάβος αββακούμ αβελάδες
 αβελλίνο αβελονήσια αβετσάνο αβησσυνία αβιλές αβιλιάνο αβινιόν αβράνς αβραάμ
 αγάθη αγάπη αγάπη αγάπι αγάπιος αγήνωρ αγία βαρβάρα αγία λουκία
 αγία τριάδα αγία τριάς αγαθάγγελος αγαθή αγαθοκλής αγαθονήσι αγαθονίκη
 αγαθόνικος αγαθόπολις αγαμέμνων αγαπήνωρ αγαρηνός αγαύη αγγέλα αγγέλικα αγγέλω
 αγγελάριος αγγελίκα αγγελίνα αγγελική αγγλία αγγλικανική εκκλησία αγερσανί
 αγιά αγιαμόντε αγιαννιώτικο αγιασσός αγιος σεραφείμ αγιούλα αγιωργίτικο
 αγκίστρι αγκιλάρ αγκνώ αγκριτζέντο αγκόλα αγλαΐα αγνή αγνοδίκη αγοράκριτος
 αγοραστός αγρίνιο αγριδάκι αγριππίνα αγχίσης αγόρης αγόρω αδάμ αδάμος
 αδαμαντία αδελία γη αδρήστη αδριανή αδριανούπολη αδριανός αδριατική αδόλφος
 αετοφωλιά αετούσα αετός αζάλας αζάν αζάνας αζαρίας αζερμπαϊτζάν αηδάνι αηδόνια
 αθήναι αθήρι αθανάσης αθανάσιος αθανασία αθηνά αθηναΐς αθηναγόρας αθηνόδωρος
 αιγάλεω αιγέας αιγαίο αιγαίο πέλαγος αιγαλιώτισσα αιγεύς αιγιάλη αιγιώτισσα
 αιγός ποταμοί αιδέσιος αιδηψός αιθέριος αιθιοπία αικατερίνη αιμίλιος αιμιλία
 αινείας αιξ-αν-προβάνς αιξ-λε-μπαιν αιολία αισχύλος αιτωλία αιτωλικό
 ακάδημοι ακάδημος ακάκιος ακίνδυνος ακίτα ακαδημία ακαθή ακαμάτης ακαμπίρα
 ακαρνανία ακούνε ακράτα ακριβή ακρόπολη ακτή ελεφαντοστού ακταίος
 ακτινοπτερύγιοι ακτινόποδα ακόλουθος ακύλας αλ κασράυν αλ καφ αλ μαντίγια
 αλάσκα αλέκα αλέκος αλέξανδρος αλέξανδρος ο μέγας αλέξης αλέξια αλέξιος αλή
 αλίαρτος αλίκη αλίνα αλαά αλαγόν αλαδινό αλαμάνα αλαμαννιά αλαμπάμα αλβέρκα
 αλβανία αλβιών αλβόχ αλγεθίρας αλγερία αλγκέρο αλεξάνδρα αλεξάνδρεια αλεξία
 αλευροσάκι αλευρώδη αλθίρα αλιβέρι αλικάνδρος αλικάντε αλικαρνασσός αλκάναρ
 αλκαίος αλκαμένης αλκανίθ αλκαντάρα αλκιβιάδης αλκινόη αλκμάν αλκμήνη αλκυόνη
 αλλάχ αλμάγκρο αλμάντα αλμάτι αλμαζάν αλμερία αλμολόνγκα αλμπέρ αλμπί αλμυρός
 αλουμινίτης αλσώ αλτίνο αλταμούρα αλυσίδα αλφάρο αλφρέδος αλωνάρης αλόγια
 αλόννησος αλόνσο αλώπηξ αμάλφι αμάμι αμάντα αμάρυνθος αμάσεια αμαζόνα
 αμακούσα αμαλία αμαλιάδα αμαράντε αμαρούσιο αμαρούσιον αμαρυλλίς αματιτλάν
 αμβούργο αμβρακικός αμβρόσιος αμερική αμιένη αμισός αμμάν αμμαδιές αμμόλοχος
 αμονακλιός αμοργιανό αμοργός αμπασίρι αμπελικού αμπελόκλημα αμπού ντάμπι
 αμπούριας αμπούτζα αμπόστα αμυγδαλιά αμυκή αμφίκλεια αμφίπολη αμφιανός
 αμφιλοχία αμφιλόχιος αμφιτρίτη αμφιτρύων αμύνταιο αμύντας αμώς ανάξιππος
 ανάπολις ανάργυρη ανάργυρος ανάσταση ανάφη ανέστης ανέτα αναΐς αναγέννηση
 ανακρέων ανανίας αναξήνωρ αναξίβιος αναξίλαος αναξίμανδρος αναξίπολις
 αναξιμένης αναξιώ αναστάσης αναστάσιος αναστασία ανατάσιος ανατολή ανατολία
 ανατόλ φρανς ανατόλιος ανγκουίλα ανγκουλέμ ανδρέας ανδραβίδα ανδριάνα ανδριανή
 ανδρομάχη ανδρομέδα ανδρονίκη ανδρούλλα ανδρόνικος ανδρόφιλος ανδόρρα ανθέμιος
 ανθήκλεια ανθία ανθεστηριών ανθεστηριώνας ανθημίτσα ανθούλα ανθούσα
 ανθρωπίδας ανκόνα αννέτα αννίβας αννίτα ανναμπέλλα αννιώ αννούλα αννόβερο
 αντήνορας αντήνωρ αντίγκουα γουατεμάλα αντίγκουα και μπαρμπούντα αντίγονος
 αντίμαχος αντίμηλος αντίμπ αντίνοος αντίπαξοι αντίπαρος αντίπατρος ανταίος
 αντιγόνη αντικύθηρα αντιμάχη αντιόπη αντλία αντρέας αντρίκος αντριάνα αντρούλα
 αντωνάκης αντωνάρας αντωνία αντύπας αντώναρος αντώνης αντώνιος ανυδρίτης
 ανόι ανύσιος αξά αξιούπολη αξιώτης αομόρι αουγκούστα απάτσι απείρανθος απελλής
 αποκριά απολλωνία απολλόδωρος αποσπερίτης αποστολία αποστόλης απουλήιος
 απρίλης απρίλια απρίλιος απριλομάης απριλομάρτης απφία απόκρεω απόκρια απόλλων
 απόνησος απόστολος αράβισσα αράντ αράο αράχωβα αρέτσο αρίσταρχος αρίστη
 αρίστος αρίων αραβία αραβίδα αραθένα αρακλειά αρακλινό αραράτ αργάκι αργέαθοι
 αργίτης αργίτισσα αργεία αργείος αργείτης αργείτισσα αργεντινή αργιά αργοκοίλι
 αργολίδα αργολικός αργοσαρωνικός αργοστόλι αργυρά αργυρή αργυρία αργυρούλα
 αργυρώ αργύρης αργύριος αργύρω αργώ αρετή αρετούσα αρετσού αρζαντάν αρθούα
 αριάδνη αριάνα αριάνο ιρπίνο αριδαία αριζόνα αριστέα αριστέας αριστίων
 αριστογένης αριστογείτων αριστοκλής αριστομένης αριστονίκη αριστοτέλης
 αριστοφάνης αριστούλα αριστόβουλος αριστόδημος αριστόδικος αριστόδουλος
 αριστόμαχος αριστόνικος αριστόνους αρκάδιος αρκάνσας αρκαδία αρκεσίνη αρκοί
 αρκτούρος αρλ αρμάνδος αρμάνος αρμένιος αρμαγεδδώνας αρμαγεδών αρμαγεδώνας
 αρμενία αρμονία αρμόδιος αρνάδος αρναία αρναούτης αρνισσαίος αροάνια
 αρούμπα αρριέτα αρριανή αρριανός αρρομάνς αρσένης αρσένιος αρσενία αρσινόη
 αρτέμιος αρτεμίσιο αρτεμισία αρτεσία αρυάνα αρχάγγελοι αρχάγγελος γαβριήλ
 αρχάγγελσκ αρχέλαος αρχές αρχίλοχος αρχιμήδης αρχοντής αρχοντία αρχοντίων
 αρχόνα αρχόντω αρωμούνος αρόη ασένοβγκραντ ασήμω ασία ασίζη ασίτες ασακάβα
 ασβαάλ ασημάκης ασημένια ασημίνα ασημούλα ασημώνη ασιμπέτσου ασκληπιός ασμάρα
 ασουνσιόν μίτα ασπίς ασπασία ασπρορομπόλα ασπρούδα ασπρόπυργος ασσάμ ασσυρία
 αστέρης αστέριος αστέρω αστερούσια αστερόπη αστρινός αστροπαλιά αστρούλα
 αστυπάλαια ασφοντυλίτης ασό ασύρτικο ασώματος ατένι ατέσα αταλάντη ατζουλέτα
 ατλάντα ατλαντικός ατρέας ατροπατηνή αττάλεια αττίλας ατταλός αττική αυγέρης
 αυγερινός αυγουστής αυγουστία αυγουστίνη αυγουστίνος αυγουστιάτης αυγούλα
 αυλάκια αυλώνα αυλώνας αυξέντιος αυξίβιος αυξεντία αυρήλιος αυρηλία αυστρία
 αυστριακός αυσόνιος αφγανιστάν αφεντία αφεντούλα αφεντούλης αφθιταλίτης
 αφιόν καραχισάρ αφρικάνος αφρική αφρικανός αφροδίτη αφροξυλάνθη αχέροντας
 αχίλλειος αχίλλιος αχαΐα αχαιός αχιλλέας αϊβαλί αϊζουβακαμάτσου αϊζουμισάτο
 αϊνστάιν αϊντάχο αϊντχόφεν αϊπάτια αϊτή αϊόβα αόστη αύγουστος αύδακτος αύδατος
 βάαλ βάγγος βάγια βάδιμος βάι βάια βάιος βάκης βάκις βάκχος βάλια βάνδαλοι
 βάρνα βάσια βάσκος βάσος βάσσος βάστο βάσω βάφης βάφτρα βέλγιο βέλικο τάρνοβο
 βέλος βέρα βέρνη βέροια βέτσλαρ βέφα βίβιαν βίβλος βίγια νουέβα βίγο βίδο
 βίκη βίκτορας βίκτωρ βίκτωρας βίκυ βίλα ντο κόντε βίλα ντο μπίσπο
 βίλα νόβα ντε φαμαλικάο βίλα ρεάλ βίλα ρεάλ ντε σάντο αντόνιο βίλνα βίλνιους
 βίντιν βίρτσμπουργκ βίτσι βαΐα βαΐτσα βαβέλ βαβυλωνία βαβύλας βαγγέλης βαγγέλω
 βαγγελίστρα βαγγελίτσα βαγγελιώ βαγδάτη βαγιά βαγιαδολίδ βαθύ βακαγιάμα
 βακχυλίδης βαλάντης βαλάντιος βαλάντω βαλένθια βαλέρια βαλέριος βαλέτα βαλασία
 βαλεντίνη βαλεντίνος βαλεριανός βαλκάνια βαλκάνιος βαλλονία βαλλόνος βαλντεμάρ
 βαλσάμης βαλτιμόρη βανέσσα βανκούβερ βανουάτου βαντούζ βαράτσε βαρίδι βαρίσεια
 βαραδάτος βαρβάρα βαρβαρούλα βαρδέα βαρδής βαρδούσια βαρθολομαίος βαρθολομιό
 βαρλαάμ βαρνάβας βαρντάκ βαρούσι βαρσαμία βαρσανούφιος βαρσιμαίος βαρσοβία
 βασάμια βασίλεια βασίλειος βασίλης βασίλω βασανίτης βασιλάκης βασιλένα
 βασιλειάννα βασιλεύουσα βασιλική βασιλικούλα βασιλικός βασιλού βασλούι βασούλα
 βατοπέδι βατοπαίδι βαυαρία βαϊθινός βαϊκάλη βαϊμάρη βεγγάλη βεγγαλέζος
 βελιγράδι βελισάριος βελισσάριος βελισσαρία βελλερεφόντης βενέδικτος βενέτικο
 βεναρδάτος βενεδίκτη βενεδικτίνη βενεζουέλα βενεράλια βενετία βενετσάνος
 βενιζέλος βερενίκη βερμόντ βερνέιγ-συρ-αβρ βερολίνο βερονίκη βερούλα βερούτιν
 βερτζαμί βερτσέλι βερτσέλλι βερόνα βερόνικα βεσσαραβία βηθλεέμ βηλάνα βηρυτός
 βησσαρίων βιάνα ντο καστέλο βιένη βιέννη βιαντάνα βιβή βιβιάνα βιβιανή
 βιγένα βιγιαρεάλ βιγιαρρομπλέδο βιδερίτης βιδιανό βιεντιάν βιετνάμ βιζέλα
 βιθυνία βικέντιος βικεντία βικιβιβλία βικιείδη βικιεπιστήμιο βικιθήκη
 βικινέα βικιπαίδεια βικιφθέγματα βικτωρία βιλιάλμπα βιναρόθ βινιόλα βιολέτα
 βιρμανία βιρτζίνια βιστούλας βισύ βιτέρμπο βιτεμβέργη βιτρέ βιτσέντσα βιτόρια
 βλάσης βλάσιος βλάσσιος βλάχικο βλάχος βλαδίμηρος βλαδιβοστόκ βλασία βλαχία
 βογκέρα βοδενά βοημία βοθύλακας βοιωτία βοκολίδα βολίνη βολίτσα βολιβία
 βολτέρα βονιφάτιος βοσνία βοσνία και ερζεγοβίνη βοσνία-ερζεγοβίνη βοστίτσα
 βουβών βουδαπέστη βουκουρέστι βουλγάτα βουλγαρία βουλγκάτα βουλώνη βουρβουριά
 βουρλιώτες βοϊβοντίνα βοϊδομάτης βοόζ βούδας βούλα βούλγαρη βούλης βούπερταλ
 βοώτης βρέλος βρέμη βρέστη βραΐλα βραδυανό βραζιλία βρασίδας βραχνός βραχώρι
 βρετανίδα βρετανική κολομβία βρισηίς βροντίνος βροτέας βρουτία βρούτης
 βρούτσης βρυξέλλες βυζάντιο βυζαντινός βυτίνα βωμός βόλγας βόλγκογκραντ βόλος
 βόλφανγκο βόνιτσα βόννη βόρας βόρεια αμερική βόρεια ιρλανδία βόρεια καρολίνα
 βόρεια ντακότα βόρις βόσνιος βόσπορος βύβλος βύρων βώλακας βώλος βώνη γάιδαρος
 γάνδη γάσπαρης γέγλες γέιδα γέκλα γένοβα γέρα γέστε γήινος γίδες γαία γαβρίλος
 γαβριήλ γαδουριά γαλάτεια γαλήνη γαλακτία γαλακτίων γαλανάδο γαλαξίδι
 γαλατία γαληνοτάτη δημοκρατία της βενετίας γαληνός γαλικία γαλιόττα γαλιώτα
 γαλλικά νότια και ανταρκτικά εδάφη γαλλική γουϊάνα γαλλική πολυνησία γαλλικός
 γαμηλιών γανυμήδης γαργαλιάνοι γαρουφαλλιά γαρούνας γαρυφαλλιά γαστούνη
 γαυριονήσια γαϊανή γαϊανός γαϊτάνος γαύδος γαύριο γείτων γεβράσιος γεδεών
 γενάρης γενεβιέβη γεννάδιος γενοβέφα γεράνεια γεράνι γεράσιμος γερακίνα
 γερασιμούλα γερμανία γερμανός γεροντίνη γερόλακκος γεσθημανή γεωργάκης
 γεωργάς γεωργέας γεωργής γεωργία γεωργογιάννης γεωργουλάκος γεωργουλέας
 γεωργούλης γεώργης γεώργιος γη γηρυόνης γιάγκος γιάγκουρας γιάλοβα γιάμε
 γιάνναρος γιάννενα γιάννεσα γιάννης γιάννινα γιάννος γιαγκούλας γιακουμής
 γιαλούσα γιαμάγκα γιαμάντα γιαμαγκάτα γιαμαγκούτσι γιαμαμότο γιαματσούρι
 γιαναΐζου γιαναγκάβα γιανγκτσέ γιαννάκης γιαννέσα γιαννακός γιανναράς
 γιαννιός γιαννιώ γιαννουλάκος γιαννούλα γιαννούλης γιαννούρης γιασεμή γιασεμής
 γιαχάμπα γιαχβέ γιβραλτάρ γιερεβάν γιεχωβά γιοβάν γιοβάννα γιοβάννης γιοζγκάτ
 γιοκότε γιολάντα γιουγκάβα γιουγκοσλάβος γιουγκοσλαβία γιουζάβα γιουκατάν
 γιουριχότζο γιούδας γιούλα γιούμπαρι γιούνης γιούρι γιούρογκρουπ γιούτα
 γιωρίκας γιωργάκης γιωργάρας γιωργία γιωργίκας γιωργίτσα γιωργίτσης γιωργούλης
 γιώργης γιώργιος γιώργος γιώργος σεφέρης γιώτα γιώτης γκάμπια γκάνα γκάντζι
 γκάρι κούπερ γκίσι γκίφου γκαέτα γκαζνί γκαλάτσι γκαλατίνα γκαμήλα γκαμπόν
 γκατζολία γκελενίτης γκελζενκίρχεν γκιλγκαμές γκιμαράες γκιοζ-τεπέ
 γκντανσκ γκολφίνος γκοούρ γκοράνος γκοσογκαβάρα γκουαλάν γκουανταλαχάρα
 γκουασακαπάν γκουαστάλα γκουτζαράτ γκουϊντόνια γκούμπιο γκράβαρα γκράντο
 γκρανβίλ γκρατς γκρενόμπλ γκροσέτο γκρόζνυ γκόα γκόγκος γκόλφω γκότο γλάδστων
 γλασκώβη γλαύκη γλαύκος γλυκέριος γλυκερία γλυκοφιλούσα γλυμής γλυνός γλυφάδα
 γλύπτης γνήσιππος γνωσίφορος γνώμων γοδεφρείδος γολγοθάς γολγοθάς γολιάθ
 γοράσδος γοργίας γοργώ γουαδελούπη γουατεμάλα γουιάνα γουινέα γουινέα μπισσάου
 γουλιέλμος γουμένισσα γουστολίδι γραμβούσα γραμματίνα γραμπονήσι γρανάδα
 γρενάδα γρηγορία γρηγόρης γρηγόριος γροιλανδία γυάρος γυαλάδικα γωγούλα γωγώ
 γόρδιο γόρης γύθειο γύλιππος γύψου γώνιτες δάκης δάμος δάμων δάρδανος δάφνη
 δέσποινα δέσπω δήλιος δήλος δήμητρα δήμος δίας δίδυμοι δίκτυον δίκωμο δίστομο
 δίων δαλμάτιος δαμαλάς δαμαριώνας δαμασκηνή δαμασκηνός δαμασκός δαμιανή
 δαμοκλής δαμοφίλη δαμοφών δανάη δανία δανακός δαναός δανιήλ δανιμαρκία
 δαρδανέλια δαρδανέλλια δαρεία δαρείος δασά δαυίδ δαφνί δείμος δεβόρα δεδουσία
 δεινοκράτης δεινόστρατος δεκέλεια δεκέμβρης δεκέμβριος δεκατρίαστρη δελφίν
 δελφοί δενούσα δεξίμαχος δεσποινιώ δεσποινούλα δεσπούλα δευκαλίων δευτέρα
 δηιδάμεια δημάρατος δημήτρης δημήτριος δημητράκης δημητρίου δημητριάδης
 δημητρόπουλος δημητρός δημιουργία δημιουργός δημοκίδης
 δημοκρατία της κίνας δημοκρατία του κονγκό δημοκρατικό δημοκρατιάτο του κονγκό
 δημοφίλη δημοφών δημοχάρης δημούλλα δημόκριτος διέππη διαβήτης διαγόρας
 διαλεχτή διαμάντω διαμαντής διαμαντίνα διασκορπισμένα νησιά στον ινδικό ωκεανό
 διγενής ακρίτας διδυμότειχο διεθνές νομισματικό ταμείο
 διεθνής φωνητική ένωση διηάνειρα διηδάμεια δικαία δικαίαρχος δικαίος διογένης
 διομήδης διονυσία διονυσούλα διονύσης διονύσιος διοσκουρίδης διοτίμα διωξίππη
 διόκλητος διόνυσος διόριος διόσκουρος διώνη δνείπερος δομένικος
 δομινίκα δομινίκη δομινικανή δημοκρατία δομιτίλλα δονάτα δονάτος δονούσα
 δοράς δορύλαιον δουβλίνο δουνκέρκη δούκας δούκισσα δούναβης δράκος δράκων
 δρέσδη δραγατσάνι δραγούμης δραγωνέρα δρακούλης δρακόντιος δροσιά δροσούλα
 δρόσος δρόσω δυνάμεις δυτική βιρτζίνια δυτική σαχάρα δωδεκάημερο δωδεκάμερο
 δωδεκάνησος δωδεκαήμερο δωδεκανήσια δωδεκανησία δωδώνη δωροθέα δωρούλα δωρούσα
 δόμνα δόξα δύση δώρα δώρος εβέλλιος εβίτα εβελίνα εβρεμούνδιος εγγαρές
 εγκέλαδος εγκαρές εδέμ εδιμβούργο εδουάρδος ειδομένη ειλείθυια ειρήναρχος
 ειρηναίος ειρηνικός εκάβη εκάλη εκάτη εκαταίος ελ αλία ελ αντελάντο ελ γκρέκο
 ελ εστόρ ελ μαντσόν ελ προγκρέσο ελ σαλβαδόρ ελένη ελίζα ελίκη ελασσόνα
 ελβίρα ελβετία ελεάνα ελεάννα ελεημονήτρια ελενίτσα ελενιώ ελεονώρα ελευθέρης
 ελευθερία ελευσίνα ελισάβετ ελισσαίος ελισώ ελλάδα ελλάδιος ελλάνικος ελλάντα
 ελλήσποντος ελληναφρικανός ελληνικοί εσπερινοί ελληνιστική περίοδος ελληνόπη
 ελπιδοφόρος ελπινίκη ελσίνκι εμμανουέλα εμμανουήλ εμπέτσου εμπίνο εμπεδοκλής
 ενετία εξάς εξακουστός εξαυδρίτης εξκάλιμπερ εξουσίες εξτρεμαδούρα εξωμιά
 επίζηλος επίκουρος επίκτητος επαμεινώνδας επανομή επιμενίδης επισκοπείο
 επιτάφιος επιφάνειος επιφάνια επιφάνιος επονίτης επονίτισσα εποχή του λίθου
 επτάνησος επταετία επτακώμη επτανήσιος επώνυμος ερένθια ερέτρια ερασμία
 ερατοσθένης ερατώ ερβάς εργότιμος ερεχθεύς ερζερούμ εριέττα ερινύα ερινύς
 ερκυνίτης ερμής ερμαίος ερμείας ερμιόνη ερμογένης ερμοκράτης ερμούπολη
 ερμόλαος ερμύλλη ερμύλλος ερρίκος ερριέτα ερυθραία ερυθρός σταυρός ερυξίμαχος
 ερωτόκριτος ερωφίλη ερύμανθος εσθήρ εσθονία εσθονός εσκισεχίρ εσκοριάλ
 εσμεράλδα εσπίνιο εσπερία εσπερίδες εστέλια εστία εσταυρωμένος εστιαιώτιδα
 ετεοκλής ετριγγίτης ετρούσκοι ευάγγελος ευάνθης ευέλθων ευέλπιστος ευαγγελή
 ευαγόρας ευανθία ευβοϊκός ευγένης ευγένιος ευγενία ευγενίτσα ευγενούλα
 ευδοκία ευδοκίτσα ευδοκούλα ευδοξία ευδοξούλα ευδόκιμος ευδόξιος ευδώρα
 ευθαλία ευθαλίτσα ευθαλιώ ευθυκλής ευθυμία ευθυμίδης ευθυμούλα ευθύδημος
 ευθύμιος ευθύνους ευθύφρων ευκλεάς ευκλείδης ευκράτης ευλάλιος ευλάμπης
 ευλάμπω ευλαλία ευλαμπία ευλογία ευλόγιος ευμένιος ευμενία ευμενίδες ευμορφία
 ευπρέπιος ευπραξία ευπρεπία ευρήλιος ευρασία ευριλία ευριπίδης ευρυβιάδης
 ευρυδίκη ευρυμέδοντας ευρυμέδων ευρυνόμη ευρυπία ευρυσάκης ευρυσθεύς ευρυτανία
 ευρωομάδα ευρωπαϊκή ένωση ευρύκλεια ευρύλοχος ευρύνομος ευρώπα νήσος ευρώπη
 ευσεβία ευσεβούλα ευστάθιος ευσταθία ευσταθούλα ευστράτης ευστράτιος ευστρατία
 ευτέρπη ευτρόπιος ευτυχία ευτυχίτσα ευτυχιανός ευτυχούλα ευτύχης ευτύχιος
 ευφράσιος ευφράτης ευφραίμ ευφραίμης ευφραιμία ευφροσύνη ευφρόσυνος ευψυχία
 εφραίμ εχίνος εψοµίτης εωσφόρος εύα εύβοια εύγριπος εύδημος εύδοξος εύη εύηνος
 εύμαχος εύμηλος εύνομος ζάγκρεμπ ζάκης ζάκυνθος ζάλογγο ζάμπια ζάρκο ζάχος
 ζήνα ζήνων ζήσης ζήσιμος ζήσος ζαΐρ ζαΐρα ζαγανιάρης ζαγκουάν ζαγορά ζακχαίος
 ζαλάου ζαλοβίτικο ζαμπέτα ζαμπία ζαν πιαζέ ζαννέτα ζαννέτος ζαννής ζαννίνο
 ζαρατούστρα ζαρλάθιος ζας ζαφείρα ζαφείρης ζαφείριος ζαφείρω ζαφειρία ζαχάρω
 ζαχαρίας ζαχαρίτσα ζαχαρούλα ζεβεδαίος ζευγολατειό ζευς ζεύξιππος ζηνοβία
 ζηνόδοτος ζηνόδωρος ζησούλα ζητούνι ζιμπάμπουε ζιτούνι ζλάτα ζλάταν ζλάτκο
 ζουμιάτικο ζουμπουλία ζοφιήλ ζυγός ζυρίχη ζωή ζωίτσα ζωγραφιά ζωζώ
 ζωροάστρης ζωσιμάς ζωτικός ζόλταν ζώης ηγήμων ηγήσανδρος ηγήσιππος ηγίας
 ηγερία ηγησίας ηγουμενίτσα ηδύλη ηλέκτρα ηλίας ηλεάνα ηλεία ηλιάδα ηλιάκος
 ηλιάνης ηλιοστάλακτη ηλιόδωρος ηλιόφωτος ηλιόχαρη ημαθία ημιυδρίτης ηνίοχος
 ηνωμένα αραβικά εμιράτα ηνωμένες πολιτείες ηνωμένες πολιτείες της αμερικής
 ηράκλεια ηράκλειες στήλες ηράκλειο ηράκλειτος ηρακλής ηρακλίτσα ηρακλειά
 ηρωδίας ηρωδίων ηρωδιάδα ηρόδοτος ηρώ ησίοδος ησαΐας ησιόνη ησύχιος ηωήλ ηώ
 θάνος θάσος θάσος θέκλα θέλμα θέμης θέμιδα θέμις θέουτα θέρμεια θέρμο θέτις
 θήρα θίμφου θαλάσσιος θαλής θαλλέλαιος θαλλής θαμόρα θανάσης θανάσιος θανασία
 θαυµασίτης θεάνθρωπος θεή θεαγένης θεανώ θειακό θελξιώπη θεμέλιος θεμιστοκλής
 θεοβάλδος θεογεννήτωρ θεοδοσία θεοδούλα θεοδούλη θεοδόσης θεοδόσιος θεοδότη
 θεοδώριχος θεοκλής θεοκυήτωρ θεολογία θεολόγης θεολόγος θεομήτωρ θεομακάριστος
 θεονύμφη θεοπίστη θεοπούλα θεοσέβιος θεοτίμη θεοτόκης θεοτόκος θεοφάνεια
 θεοφάνης θεοφάνια θεοφίλη θεοφανία θεοφανώ θεοφύλακτος θεοχάρης θεράπων
 θερμαϊκός θερμιά θερσίτης θεσπιές θεσπρωτία θεσσαλία θεσσαλονίκη θεσσαλός
 θεωνάς θεωνίτσα θεόδοτος θεόδουλος θεόδωρος θεόκλητος θεόκριτος θεόπεμπτος
 θεόπιστος θεός θεότεκνος θεότιμος θεόφιλος θεόφραστος θεώνη θηλαλιευτής
 θηρασία θηρασιά θηρεσία θησέας θησείο θιβέτ θοδωράκης θοδωρής θοδωρούλα θοδώρα
 θολόγης θορικός θουέρα θουκυδίδης θράκη θράσος θράψα θρακικό πέλαγος
 θρασύμαχος θραψαθήρι θρόνοι θωθ θωμάκης θωμάς θωμαΐς θωμαή θωρ θωτ θόδωρος
 ιάκωβος ιάμβλιχος ιάπωνας ιάσιο ιάσων ιέρνη ιακωβίνα ιακώβ ιαλυσσός ιανουάριος
 ιαπωνία ιαροισίτης ιαροσλάβος ιασώ ιαχβέ ιβάκι ιβάνα ιβάνγκοροντ ιβάτε
 ιβανούμα ιβαϊζούμι ιβρέα ιγκλέζιας ιγνάτιος ιγνατία ιδομενεύς ιεοβά
 ιερά οδός ιερά σύνοδος ιεράπολη ιερεμίας ιερισσός ιεροκλής ιεροσόλυμα
 ιερόθεος ιερώνυμος ιεσθημανή ιεχοβά ιεχωβά ιζέρνια ιζαμπέλλα ιζαμπώ ιζουμιζάκι
 ιησούς ιησούς του ναυή ιθάκη ιιζούκα ιιτάτε ικάβα ικάριο πέλαγος ικαρία
 ικόνιο ιλάριος ιλαρίων ιλιάδα ιλιάς ιλινόι ιλλµενίτης ιλλίτης ιλλουμινάτοι
 ιλλυριός ιλλύριος ιμάρι ιμαλάια ιμπέρια ιμπουσούκι ιναβασίρο ινγλάνδη ινδία
 ινδοευρωπαία ινδοευρωπαίος ινδονησία ινδός ιντιάνα ιντιανάπολις ιντούν
 ινώ ιξίων ιοκάστη ιορδάνης ιορδανία ιουλία ιουλίτα ιουλίτη ιουλίτσα ιουλιέττα
 ιουλιανός ιουνία ιουστίνη ιουστίνος ιουστινιανός ιούδας ιούδας ο ισκαριώτης
 ιούλιος ιούνης ιούνιος ιππάριον ιππίας ιπποδάμεια ιπποκράτης ιππολύτη
 ιράκ ιράν ιρκούτσκ ιρλανδία ισίδωρος ισαάκ ισαβέλλα ισαμπάλ ισαυρία ισαχάγια
 ισιδώρα ισινομάκι ισλαμαμπάντ ισλανδία ισμήνη ισπανία ισραήλ ισραήλ ιστία
 ιτέα ιταλία ιτοσίμα ιτσικικουσικίνο ιτσινοσέκι ιτσινόχε ιφιγένεια ιφικράτης
 ιχθύες ιχθύς ιπτάμενος ιχθύς νότιος ιχιά ιωάννα ιωάννης ιωάννης ο χρυσόστομος
 ιωακείμ ιωλκός ιωνάθαν ιωνάς ιωνία ιωνική δωδεκάπολις ιωσήφ ιωσηφίνα ιόδοκος
 ιόνια νησιά ιόνιο πέλαγος ιώ ιώβ κάδιθ κάδμος κάθερες κάιρο κάλι κάλια κάλιαρι
 κάλντας ντα ραΐνια κάλυμνος κάλχας κάμα κάμειρος κάμινος κάμπις κάμπος κάνδιδα
 κάνσας κάπρι κάρλοβο κάρμεν κάρμι κάρντιφ κάρολος κάρπαθος κάρπος κάρτζαλι
 κάσελ κάσος κάσσανδρος κάσσαρο κάσσυ κάστελ φρεντάνο κάστορας κάστρο
 κάστωρ κάτια κάτω απρόβατο κάτω κατάκοιλος κάτω ποταμιά κάτω σαγκρί κάφσα κάχι
 κέιπ τάουν κέλλυ κέλσιος κέμνιτς κέμπριτζ κένταυρος κένυα κέραλα κέρκυρα κέρος
 κήρυκος κήτος κία κίδραμος κίεβο κίελο κίκα κίμωλος κίμων κίνα κίναρος κίνκι
 κίτσα καέν καίσαρ καίσαρας καίτη καβάλα καβάφης καβαμάτα καβαούτσι καβυλία
 καγκαμιίσι καγκοσίμα καδής καζάν καζάρμα καζάφανι καζέρτα καζακστάν
 καζαλμπορντίνο καζανλούκ καζαχστάν καζούνο καζόλι καθαρά δευτέρα καθολική
 καικιλία καισάρεια καισάρεια καισάριος καισαριανή κακοτρύγης κακούντα
 καλή καλίνινγκραντ καλαί καλαβρία καλαγκάθι καλαμάκι καλαμάτα καλαμαριά
 καλανδίων καλαντός καλαράσι καλιφόρνια καλλίνικος καλλίστη καλλίστρατος
 καλλικλής καλλικράτης καλλινίκη καλλιοπίτσα καλλιπάτειρα καλλιρόη καλλισθένη
 καλλιόπη καλλιόπιος καλλονή καλογραία καλοδότη καλομοίρα καλοταρίτισσα
 καλούδι καλυβάρι καλυψώ καλόξυλος καμένα βούρλα καμερούν καμηλοπάρδαλις
 καμικοάνι καμουήλ καμπανία καμπούλ καμπότζη κανάζαβα κανάκης κανάκι κανέλλα
 κανέλος καναανίτης καναδάς κανακάρι κανδελάρια κανεγιάμα κανεγκασάκι κανζάκι
 καντέρμπερι κανταχάρ καντελέντα καντό κανόγια καολινίτης καπίσα καπίτων
 καποδίστριας καπούτι καππαδοκία καππαριά καράκουμι καράρα καράτσου καρία
 καραβάς καραβάτζιο καραβοστάσι καραγκιόζης καρακάλλας καραμάν καραμανία
 καραντάν καραντίνα καραντόκιος καρατάσι καραϊβική καρδίτσα καρδεδέου καρδιανή
 καρκίνος καρκασσόν καρλσρούη καρλόβασι καρμανιόλα καρμπάγια καρμπάγιο
 καρμόνα καρνάτακα καρναχαλάς καρολίνα καρολομάγνος καρουμάι καρπάθιο
 καρπάθιος καρπάσιος καρπαθία καρπασία καρπενήσι καρράρα καρς καρτέγιος
 καρυά καρυές καρχηδόνα καρύανα κασίμα κασίνο κασκάις κασμίρ κασούγκα κασπία
 κασσάνδρεια κασσιανή κασσιανός κασσιόπεια κασσιόπη καστέλλι καστέλο μπράνκο
 καστελβετράνο καστελλόριζο καστοριά καστούλος κατάκοιλος κατάκολο κατάνη
 κατάπολα κατάρ κατέρω κατίνα καταγκάμι καταγλάν κατακαλαίοι καταλονία
 καταλωνία καταντζάρο καταρόχα κατερίνα κατερίνη κατερινάκι κατερινούλα
 κατμαντού κατοβίτσε κατοχή κατούνα κατσανό κατσιμπούχαιροι κατσούνι καττίδιος
 κατωκοπιά καφριστάν καϊμακτσαλάν καϊρουάν καύκασος κελερίνος κελεστίνος κεμάλ
 κεμπεκουά κενδέας κενσούριος κεντάκι κεντρική αμερική
 κεραμί κεραμωτή κερασία κερασίνα κερασοχώρι κερασούντα κερατσίνι κερατσούδα
 κερύνεια κεσάδα κεσάρεια κεσαλτενάνγκο κεσενούμα κεφαλλονιά κεφαλονιά
 κηφέας κηφεύς κιάδος κιάρα κιάτο κιβάσο κιζερίτης κικέρων κική κικίτσα
 κιλελέρ κιλικία κιλκίς κιμπιλί κινσάσα κιοστέμ κιουσέμ κιουστεντίλ κιουτάχεια
 κιρίμπατι κιργιζία κιργιστάν κιριμπάτι κιρόγα κισινάου κιτάμι κιτίμ κιταακίτα
 κιτακάτα κιτακυούσου κιτασιομπάρα κιτσέ κιτσούκι κιττία κιόνια κιότζα κιότο
 κλέανδρος κλέαρχος κλέλια κλέων κλήμης κλήνης κλαίλια κλαίρη κλαζομενές
 κλαύδιος κλεάνθης κλείτος κλειδώ κλεινοβός κλεισίδι κλεισθένης κλειτόδωρος
 κλεομένης κλεονίκη κλεοπάτρα κλεπίνη κλεόβουλος κλεόνικος κλεόπας κλημέντιος
 κλινοβός κλοτίλδη κλουζ-ναπόκα κλυταιμνήστρα κλωθώ κλώντια κνίδος κνωσός κογκό
 κοζέντσα κοινωνία των εθνών κοκκινιά κοκκινοσκουφίτσα κολίνος κολινδρινό
 κολλινιάτικο κολμάρ κολοκοτρώνης κολομβία κολομπύρι κολοράντο κολούμπια
 κολχίδα κολωνία κολόμπους κολώνα κομισιόν κομμαγηνή κομνηνός κομοτηνή κομπάν
 κομπότι κομφούκιος κομόρες κονέκτικατ κονγκό κονδήλω κονδυλένια κονδυλία
 κονελιάνο κονκλάβιο κοντέ αν μπρι κοντήλω κοντακαίικα κοντεμένος κοντοκλάδι
 κοντυλιά κοντυλιά κοντύλω κοπάρο κοπεγχάγη κοράνιο κορέα κορέλια κορίνα
 κορδελιό κοριγιάμα κοριλιάνο κοριλιάνο καλάμπρο κορινθία κορινθιακός
 κορνέτο κορνήλιος κορνηλία κορσική κορτεγκάνα κορτόνα κορώνεια κοσάκα κοσμάς
 κοστούμ κοτσιφάλι κοτσιφαλιάτικο κοτσιφολιάτικο κοτσορώνης κοτύωρα κουάκερος
 κουένκα κουβέιτ κουβέλτσι κουζουμάκι κουζουνός κουιλάπα κουκλουτζάς κουλίτσα
 κουμάρος κουμαμότο κουμανή κουμελάς κουνάρ κουνίμι κουνισάκι κουντούζ κουνόχε
 κουρασάο κουριχάρα κουροΐσι κουρου-τσεσμέ κουρουνοχώρι κουρούμε κουρούπης
 κουρσκ κουρτζολάρι κουσίμα κουσίρο κουτάνς κουτσοβέντης κουφονήσια κοϊντίνος
 κούλα κούλουρη κούνεο κούρδισσα κούρδος κούτζι κούτσι κράτος της παλαιστίνης
 κρήνη κρήτη κρίστυ κρίτων κρακοβία κρανίδι κρασάτο κρατήρ κρατύλος κραϊόβα
 κρηνιώ κρης κρητηνία κρητικό κρητικό πέλαγος κριός κροίσος κροατία κροζέ νήσοι
 κρουστάλλω κρυστάλλης κρυστάλλω κρυσταλλία κρυσταλλοπηγή κρυόβρυση κρόκος
 κρόνος κρότωνας κτησιφών κτικάδος κυβέλη κυδωνίες κυδωνίτσα κυθέρεια κυθραία
 κυούσου κυπαρισσία κυπαρισσιακός κυπρί κυπρία κυπριανή κυπριανός κυρά κυρήνη
 κυριαζής κυριακή κυριακή της ανάστασης κυριακή των βαΐων κυριακούλα κυριότητες
 κυότο κχοστ κωλοκοτρόνης κωμιακή κωνστάντζα κωνσταντής κωνσταντία κωνσταντίνα
 κωνσταντινάκης κωνσταντινιά κωνσταντινούπολη κως κωστάκης κωστής κωσταντία
 κωσταντίνος κωσταντινάκης κωστούλα κόγκα κόιμπρα κόιντος κόλυμπος
 κόμο κόμπε κόναβος κόνιτσα κόνκορντ κόνων κόραξ κόρδοβα κόρθι κόρινθος κόρωνος
 κόστα ρίκα κότσι κόφου κόχυλος κύθηρα κύθηρος κύθνος κύκνος κύμη
 κύνθια κύπρια κύπρος κύρα κύριλλος κύριος κύρκος κύρος κύρρος κύων μέγας
 κώμη κώνστας κώστας κώτσος λ' άκουιλα λάγκος λάζαρος λάιος λάκης λάμνα
 λάμπης λάμπρος λάμυρα λάνσελοτ λάντσχουτ λάος λάουρα λάπαθος λάπηθος λάρητες
 λάρνακα λάσκαρης λάτιο λάχεση λέανδρος λέκο λένα λένας λένος λέο τολστόι λέρμα
 λέσβος λέτα λέτσε λέων λέων μικρός λήδα λήμνος λία λίας λίβανος λίβερπουλ
 λίγηρας λίγυς λίζα λίλιθ λίμα λίμπεκ λίνα λίνδος λίνκολν λίνος λίπαρι λίτσα
 λα εστράντα λα κορούνια λα ροσέλ λα σπέτσια λαβέλο λαγκαδάς λαγκμάν λαγωός
 λαδικινό λαιστρυγόνες λακεδαίμονας λακωνία λακωνικός λαλάκης λαμία λαμπέτι
 λαμπρινή λαμπρινός λανουσέι λανς λαντσιάνο λαοδάμας λαοδίκη λαοκράτης
 λαοκόων λαουρία λαούτη λαπωνία λαρέδο λαρίνο λαρίσα λας βέγκας λασά λασίθι
 λαυρέντης λαυρέντιος λαυρεντία λαϊκή δημοκρατία της κίνας
 λαϊκή δημοκρατία του τιμόρ-λέστε λαύρα λαύριο λεία λεβίδι λεβερκούζεν λειψία
 λεμεσός λεμονιά λενάκι λενιάνο λενιώ λεονάρδος λεονάρισσο λεοντάνθρωπος
 λεοντοβούνια λεοντοκέφαλος λεοντόθεος λεονφόρτε λεπτοκαρυά λεσόθο λεττονία
 λευθέριος λευκάδα λευκή λευκοθέα λευκορωσία λευκορώσος λευκούλα λευκωσία
 λευτέρης λευτέριος λευτεράκης λευτεριά λεχία λεωκράτης λεωνάρδος λεωνίδας
 λεωχάρης λεϊρία λεόντιος λεύκα λεύκες πάρου λεύκιος λεύκιππος λεώνη ληθαίος
 λημνιώνα ληξούρι λητώ λιάκος λιάνα λιάς λιάτικο λιέιδα λιβάδα λιβάδια λιβέρης
 λιβερία λιβυκό πέλαγος λιβόρνο λιβύα λιβύη λιγονέρι λιγουρέζος λιδωρίκι λιζέτα
 λιλ λιμενάρια λιμνιά λιμόζ λινάρδος λιντς λιοΐρι λιουμπλιάνα λισαβόνα λισαβώνα
 λιτόχωρο λιχτενστάιν λιψία λοΐζος λογκάρ λογοθέτης λογροσάν λονδίνο λονδῖνο
 λορέντζος λοριάν λος άντζελες λουΐζος λουάντα λουάρκα λουίζα λουζάδα λουιζιάνα
 λουκία λουκιανός λουκρητία λουλέ λουλού λουλούδα λουλούδω λουντμίλα
 λουξεμβούργο λουρουκίνα λουτρά λουτράκι λουτρός λούγο λούζα λούης λούθηρος
 λούλα λούλης λούμπα λυγαρίδια λυγερή λυγξ λυγουριό λυδία λυθράγκωμη λυκάονας
 λυκαονία λυκομήδης λυκούργος λυκόστομος λυμπέρης λυσίας λυσίμαχος λυσιστράτη
 λυών λυώνας λωξάντρα λωτοφάγοι λόβετς λόκι λόλα λόντι λόντρα λόρδος βύρωνας
 λόχα λύκος λύρα λύσανδρος λύσιππος λύτρια λώρα μάαστριχτ μάγδα μάγια μάγκιζα
 μάης μάια μάιντς μάιος μάκης μάλαγα μάλαμα μάλι μάλιε μάλτα μάμας μάνδρα μάνθα
 μάνια μάνος μάνος χατζιδάκις μάντισον μάντοβα μάνχαϊμ μάξιμος μάρα μάραγκας
 μάρθα μάριος μάρκελλος μάρκελος μάρκος μάρκος τύλλιος κικέρωνας μάρμπουργκ
 μάρτιος μάρω μάσα μάσα μαρίτιμα μάτα μάτης μάτσουε μάφρα μάχη μάχος μέγαρα
 μέθανα μέιν μέιντστοουν μέλανες μέλπω μέμα μέμος μέμφις μένανδρος μένητες
 μένιος μένιππος μέριλαντ μέριντα μέση μέση ανατολή μέση ποταμιά μέστρε μέτσοβο
 μήλος μήτρος μήτση μήτσος μία μίθρας μίκα μίκης μίκης θεοδωράκης μίλητος
 μίλτων μίλχαϊμ μίμα μίμης μίνα μίνστερ μίνωας μίνως μίρκα μίρνα μίσιγκαν μίτο
 μαία μαίρη μαγγεµίτης μαγδάλω μαγδαληνή μαγδεβούργο μαγδούλα μαγιάπριλο
 μαγιότ μαγκαντάν μαγκρέμπ μαγναύρα μαγνησία μαγνησίτης μαδάρα μαδαγασκάρη
 μαδρίτη μαεμπάσι μαζαράκι μαθιός μαθουσάλας μακάο μακάρες μακάριος μακεδνό
 μακεδονία μακουραζάκι μακρίνα μακρονήσι μακροτάνταλο μακρόνησος μαλάκα μαλάκκα
 μαλάουι μαλαγουζιά μαλαισία μαλακάλ μαλαματένια μαλαμώ μαλαχίας μαλβίνα
 μαλδίβες μαλιακός μαλουκάτο μαμάδος μανάμα μανίλα μανίσα μανδηλαριά μανιπούρ
 μανούσος μαντία πραντές μαντανίγιν μαντηλάρι μαντιλάς μαντούρια μαντρακούκος
 μαντώ μανφρεντόνια μανωλιά μανόλης μανώλης μαράκι μαρία μαρίκα μαρίνα μαρίνος
 μαραθόκαμπος μαραθώνας μαργαρίτα μαργαρίτης μαργιωρή μαριάνα μαριάνθη μαριάννα
 μαριγούλα μαριγώ μαρικάκι μαριλένα μαριλίτσα μαριούπολις μαριωρή μαριώ μαριώλα
 μαρκέλλα μαρκιανή μαρκιανός μαρλάς μαρλέν ντίτριχ μαρμαράς μαρουλίτσα μαρουλιώ
 μαρουσιώτης μαρουσιώτισσα μαρουσώ μαρούλα μαρούσα μαρούσι μαρσάλα
 μαρτινίκα μαρόκο μασάουα μασάφρα μασαθούρι μασατενάνγκο μασαχουσέτη μασσαλία
 ματέρα ματίλντα ματίνα ματζέντα ματθίλδη ματθαίος ματούλα ματρώνα ματσαρίνο
 ματσουγιάμα ματσουούρα ματσουσίμα μαυράθηρο μαυρέτα μαυρίκιος μαυριτανία
 μαυροβούνιο μαυρογένης μαυρογένους μαυρογιάννης μαυρογιώργος μαυροδάφνη
 μαυροκορδάτος μαυροκορδάτου μαυροκουντούρα μαυρομάτης μαυρομιχάλης μαυρομμάτης
 μαχαράστρα μαύρα μαύρος μαύρου μαῦρος μεβλεβήδες μεγάλη άρκτος μεγάλη βρετανία
 μεγάλη εβδομάδα μεγάλη πέμπτη μεγάλη παρασκευή μεγάλη πλάκα μεγάλη σαρακοστή
 μεγάλη τετάρτη μεγάλη τρίτη μεγάλο γαυριονήσι μεγάλο σάββατο μεγακλής
 μεγαλόπολη μεγαλόχαρη μεγασθένης μεγκαλάγια μεζάνιε μεθόδιος μελάνη μελέαγρος
 μελένικο μελέτης μελέτιος μελίδα μελίνα μελίτα μελίτων μελανή μελανία μελετία
 μελιτίνη μελούσεια μελπομένη μενέλαος μεξικάνος μεξικό μεράνο μερίκας μερίτσα
 μερκούρης μερκούριος μερμηγκιές μερσίνη μερόπη μεσαίωνας μεσαθούρι μεσαριά
 μεσολόγγι μεσοποταμία μεσσήνη μεσσηνία μεσσηνιακός μεσόγειος μεσόγειος θάλασσα
 μετέωρο μεταξία μεταξούλα μετς μηλένια μηλίτσα μηλιές μηνάς μητροφάνης
 μιά μηλιά μιανμάρ μιγιάγκι μιγιάκο μιγιάμα μιγιαβάκα μιγιαζάκι μιγιακονοτζό
 μιζόραμ μιθριδάτης μικάσα μικέλης μικές μικρά άρκτος μικρά ασία
 μικρή άρκτος μικρασία μικροδαρδανέλλια μικρονησία μικροσκόπιον μιλάνο μιλάτσο
 μιλήσιος μιλτιάδης μιμή μιμίκα μιναμάτα μιναμί-σιμαμπάρα μιναμιαΐζου
 μιναμισανρίκου μιναμισατσούμα μιναμισόμα μινερβίνο μούρτζε μινεσότα μινσκ
 μιράντα μιράντα ντο κόρβο μιράντα ντο ντούρο μιρέλλα μιρίτσα μισάβα μισάτο
 μισισίπι μισσισσιπής μιστρέτα μιτάνε μιχάλης μιχάρου μιχαέλα μιχαήλ
 μιχαήλα μιχαήλος μιχαίας μιχαλία μιχαλίτσα μνημοσύνη μνησικλής μογγολία
 μοζαμβίκη μολάοι μολδαβία μολδοβλαχία μολιτέρνο μολφέτα μονή μονή παναχράντου
 μοναρκά μονεμβάσια μονεμβασιά μονκό-λε-μιν μονμπέτσου μονπελιέ μονρεάλε μονς
 μοντγκόμερι μοντελμόσου μοντεμόρ-ο-νόβο μοντοβί μοντρεάλη μοντσερράτ μοντωμπάν
 μονόκερως μονόπολι μοριόκα μορτάκια μορτάρα μορφέας μορφούλα μοσούλη μοσχάνθη
 μοσχοφίλερο μοσχούλα μοσχόμαυρο μοτομίγια μουδανιά μουλαντόνα μουνακάτα
 μουροράν μουσιώνας μουτσούνα μούρμανσκ μούρος μούτσου μούχταρο μπάια μάρε
 μπάμπης μπάνσκο μπάρι μπάχα βεραπάς μπέλαπαϊς μπέντζαμιν φράνκλιν μπέπου
 μπέργκαμο μπέρμιγχαμ μπέρχα μπέστλα μπέτυ μπία μπίθρος μπίλεφελντ μπίσμαρκ
 μπαγιέ μπαγκερία μπαγκλάν μπαδαχόθ μπακάου μπακού μπακράτσι μπαλάσης
 μπαλντρ μπαλχ μπαμιάν μπανγκλαντές μπανγκόκ μπανγκόρ μπανιάρα καλάμπρα
 μπαντ χόμμπουργκ μπανταχσάν μπαντγκίς μπαριάμα μπαρλέτα μπαρμπάντος
 μπαρσέλος μπαρφλέρ μπασάνο μπαστιά μπατάλια μπατίκι μπατίστας μπατζά μπατσί
 μπαχάμες μπαχρέιν μπαϊρακλί μπεάτα μπεγλέρι μπεζανσόν μπεκάρι μπεκιάρι μπελίζε
 μπεμπέκα μπενίν μπενεβέντο μπενιζέλος μπερδεμιάρος μπετζέχρω μπετόβεν μπιέλα
 μπιλμπάο μπιμπάι μπιν αρούς μπιτόντο μπιχάρ μπλαγκόεβγκραντ μπλουά μποβίνο
 μπογιαλαμάδες μπολού μπολτσάνο μπολόνια μπονόρβα μποργκομανέρο μπορντό
 μποτσουάνα μπουένος άιρες μπουζάου μπουνγκοτακάντα μπουργκάς μπουρζ
 μπουρνόβας μπουρούντι μπουτάν μπουτζάς μπούζεν μπούνγκο-όνο μπούρμα μπούφτεα
 μπρένερο μπρέσα μπρίντιζι μπρα μπραγκάνσα μπρασόβ μπρατισλάβα μπρεσανόνε μπρνο
 μπρυζ μπόζα μπόιζι μπόμπος μπότσεν μυία μυκάλη μυκήνη μυλοπόταμος μυλούζ μυούς
 μυριάνθη μυρσίνη μυρτιά μυρτώ μυρτώο πέλαγος μυσία μυσός μυτιλήνη μωάμεθ
 μωυσής μωϋσής μόδεστος μόνα μόναχο μόνικα μόντενα μόντικα μόντρεαλ μόντσα μόρα
 μόρφω μόσχα μόσχος μόσχω μύκονος μύλασα μύρινα μύρτος μύρτου μύρων μύσιος
 νάγια νάκης νάμπουλ νάντη νάντια νάξος νάουσα νάπολη νάρα νάρκισσος νάσβιλ
 νάσος νάτα νάταλι νέα μάκρη νέα ρώμη νέαρχος νέβας νέγκρετ νέλας νέο δελχί
 νέρτους νέστορας νέστωρ νέτα νήσοι αγίου πέτρου και μιχαήλ νήσοι μάρσαλ
 νήσοι σολομώντος νήσος άγιος παύλος νήσος άμστερνταμ νήταυρος νίγηρας νίκαια
 νίκη νίκος νίκων νίμος νίνα νίνος νίσυρος νίτσα ναγιόρο ναγκάνο ναγκαλάντ
 ναγκόγια ναζαρέ ναζαρέτ ναζωραίος ναθαναήλ νακάμα νακάτσου νακατζίμα ναμίε
 νανγκαράρ νανσύ ναξία σμύρις ναούμ ναούρου ναπαίος ναπολέων ναράχα ναρντό
 νατάλιος νατάσα ναταλία νατόρι ναυσικά ναϊρόμπι ναύπακτος ναύπλιο νεάντερταλ
 νεβάδα νεβέρ νεβσεχίρ νεγκόσκα νεκρά θάλασσα νεκτάριος νεκταρία νεμέα νεμούρο
 νεοκλής νεοφύτα νεοχάρης νεπάλ νερέος νεραντζιώτισσα νεσέμπαρ νεστορία νετούνο
 νεφελοκοκκυγία νεόκαστρο νεόκριτος νεόφυτος νημποριό νησιά του σολομώντα
 νιγρίτα νιιγκάτα νικ νικάνωρ νικάστρο νικάχο νικήστρατος νικήτας νικίας
 νικαράγουα νικηφορία νικηφόρος νικοκλής νικολάκης νικολέτα νικολέττα νικολής
 νικολίνα νικολίτσα νικολοβάρβαρα νικολός νικομήδεια νικουριά νικόλαος νικόλας
 νιμρούζ νινέτα νινή νινίτα νινόχε νιος νιου τζέρσεϊ νιου χάμσαϊρ νιούε
 νισιάιζου νισιβάγκα νισιγκό νισινοομότε νισύρια νισύριος νιτσινάν νιχονμάτσου
 νιόβη νιόνιος νοέμβρης νοέμβριος νοβάρα νοβοροσίσκ νοβοσιμπίρσκ νογκάτα
 νομική νομικός νομπεόκα νομποριμπέτσου νορβηγία νοσίρο νουρεστάν
 ντάισεν ντάκα ντάλια ντάνα ντάνι ντάπια ντάρμστατ ντάτε ντέλαγουερ ντέμη
 ντένβερ ντένης ντέρεμ ντίμης ντίνα ντίνος ντίνσλακεν ντίσελντορφ ντίτα
 νταίζη νταζάιφου νταραγάτσι νταρόκα ντε μόιν ντεζάνα ντελεσίμι ντεμπίνα
 ντεσάου ντεϊκούντι ντιάνα ντιγιάρμπακιρ ντιζόν ντιπ ντοναμούλα ντονιέσκ
 ντράγκα ντόμπριτς ντόνιετ ντόρα ντόργκαλι ντόρτμουντ ντόχα ντύρμερσχαϊμ
 νυμφίος νυρεμβέργη νόβγκοροντ νόντα νόρμα νόρσιπινγκ νότια αμερική
 νότια καρολίνα νότια κορέα νότια ντακότα νώε ξάνθη ξάνθιππος ξάνθος ξέναρχος
 ξένια ξαβεριώτισσα ξανθή ξανθίππη ξανθούλα ξενία ξενοκράτης ξενοφάνης ξενοφών
 ξηρόκαμπος ξινάρα ξινόμαυρο οία οίακας οβάρ οβίδιος οβδιού οβρυά ογκόρι οδέττη
 οδυσσέας οδύσσεια οθωνοί οιδίποδας οικουμένιος οινόφυτα οκάβα οκαγιάμα
 οκλαχόμα σίτυ οκούμα οκρίβας οκτάς οκτώβρης οκτώβριος ολβιανός ολγάκι
 ολιβέιρα ντε φράτες ολιβέιρα ντο όσπιταλ ολλανδία ολλανδικές αντίλλες ολυμπία
 ολυμπιοδώρα ολύμπια ολύμπιος ομάν ομούτα ομπερναί ομπερχάουζεν ομπιχίρο ομσκ
 ονήσιμος ονδούρα ονησίφορος ονιάρα ονούφριος οντάτε οντίν ονωράτος ονότζο
 οξύνεια οράντεα ορέστης οργανισμός βορειοατλαντικού συμφώνου
 ορεστιάδα οριουέλα ορλεάνη ορουζγκάν ορσαλία ορτίσιος ορτανσία ορφέας
 οσάκα οσέρ οσού οσπίτιος οστρογότθοι οτάρου οτσούτσι οττάβα ουΐτιγγης
 ουάσιγκτον ουάσινγκτον ουέλβα ουέλλινγκτον ουαγκαντουγκού ουαλέριος ουαλία
 ουαλερίνος ουαλεριανός ουαλόνος ουαϊόμινγκ ουγγαρέζα ουγγαρέζος ουγγαρία
 ουγκάντα ουεουετενάνγκο ουζμπεκιστάν ουισκόνσιν ουκίχα ουκρανή ουκρανία
 ουλλρ ουλμ ουμβέρτος ουράβα ουράνιος ουρίκε ουρανία ουρανός ουρβανός ουρεσίνο
 ουρουγουάη ουρουγουανός ουσά ουσάκ ουσούκι ουτάρ πραντές ουταράχαντ ουτασινάι
 ουτρέρα ουτρέχτη ουτσουνομίγια ουψάλα ουϊστάνος οφηλία οφιούχος οφουνάτο οχάιο
 οψίκιο οϊτίν ούαρος ούγγρος ούγκο ούκι ούλεν ούνζεν ούννος ούντινε ούρσουλα
 πάβελ πάδουα πάικος πάλμα ντε μαγιόρκα πάλμα ντε ρίο πάμπος πάνας πάνερμος
 πάνος πάνταινος πάντοβα πάργα πάρης πάρις πάρμα πάρος πάσκα πάσος ντε φερέϊρα
 πάσχα πάτμος πάτραι πάτροκλος πάφος πάχνα πέμπτη πένζα πέπη πέπος πέραμα πέραν
 πέρνικ πέρσης πέρσιβαλ πέρσιος πέτρα πέτρος πήγασος πήλιο πίζα πίκτος πίνδαρος
 πίπης πίστη πίτσα παΐσιος παβία παγασητικός παγγαία παγκράτιος
 παγώνα παιανία πακιστάν πακιστανός πακτίκα πακτιά παλάου παλέρμο παλαίκυθρο
 παλαιστίνη παλαιστού παλαιόσοφος παλαμήδης παμίδι παμίνος παμβώτιδα
 παμπιλιόσα ντα σέρα παμπόροβο παμφυλία παν πανάγος πανάρετος πανίκος παναΐλα
 παναγής παναγία παναγιά παναγιωτάκης παναγιωτούλα παναγιώτα παναγιώτης
 παναθήναια παναθηναϊκός παναμάς παναϊτής πανδώρα πανευρώπη πανηγύριος
 πανιώνιος παννότια πανορμίτης πανουλάκης πανούλης πανούσης πανούτσος
 πανταζής πανταλέων παντελάκης παντελής παντελεήμων παντζάμπ παντζίρ παντμίνι
 παντολέων παντούλα πανωραία παξοί παπαδάκης παπαδάκις παπαδόπουλος παπανδρέας
 παπούα νέα γουινέα παράσχος παρέδες παρίσι παρίσια παρίσιοι παραγουάη
 παραμυθιά παρασκευάς παρασκευή παρασκευούλα παρβάν παρθένα παρθένης παρθένιος
 παρθενία παρθενοπαίος παρθενόπη παρθενώνας παριανό παρμενίων πασχάλης πασχαλία
 πασχαλιά πατάπιος πατής πατρίκι πατρίκιος πατραϊκός πατρινέλλα παυλάκης
 παυλάρας παυλίνα παυσίας παυσίκακος παυσανίας παφλαγονία παφνούτιος παχούμιος
 παύλος παῦλος πεδεμόντιο πεζάντα πειθώ πειρίθους πειραιάς πειραιεύς
 πεισιστρατίδες πεκίνο πεκς πελάγιος πελίας πελαγία πελαγίτσα πελαγόνησος
 πελοπίδας πελοπόννησος πεναφιέλ πενθίλος πενθεσιλεία πενθεύς πενσυλβάνια
 πεντάγωνο πεντάσταυρη σημαία πεντάσταυρο πεντάτευχος πεντηκοστή περάστρα
 περίοικοι περαία περικλής περιστέρα περιστέρι περιστερά περιστερωνάρι
 περμ περού περούτζια περπινιάν περσέας περσία περσίδες περσείδες περσεφόνη
 πεσέρσκ πεσαβάρ πεσκάρα πετέν πετράκης πετρής πετρίτσι πετροκόρινθο
 πετρούλα πετρώνιος πηγάδα πηγή πηγίτσα πηγούλα πηλέας πηλεύς πηνειός
 πηνελόπη πιάτρα νέαμτς πιέρρος πικέρμι πινερόλο πιονγιάνγκ πιπίνα πιπίτσα
 πιστεύω πισωλιμνιώνας πιτέστι πιτροφός πιτσιρίκο πλάτων πλάτωνας πλέβεν
 πλαταιές πλατιά πλατυτέρα πλατύ πλαύτος πλειάδες πλειστοάναξ πλειστώναξ
 πλοπένι πλούμπω πλούταρχος πλούτωνας πλυτό πλωμάρι πλώρα ποθητή ποθητός
 πολέμαρχος πολέμων πολίτης πολιτεία πολυβώτης πολυδάμας πολυδεύκης πολυζώης
 πολυμέρης πολυμήδης πολυνίκη πολυνείκης πολυνησία πολυξένη πολυπέρχων πολυτίμη
 πολυχρονία πολυχρόνης πολυχρόνιος πολωνέζος πολωνή πολωνία πολύαιγος πολύβιος
 πολύγυρος πολύδωρος πολύζος πολύκαρπος πολύμνια πολύτιμος πολύφημος πομπήιος
 πομόνα ποντ ντε σουέρτ ποντγκόριτσα ποντομπό ποπάι ποπίτσα πορταλέγκρε
 πορτλανδίτης πορτογαλία πορφυρή πορφυρία πορφύρης πορφύριος ποσειδεών ποσειδών
 ποσειδώνι ποσιδεών πουέρτο μπάριος πουέρτο ρίκο πουατιέ πουλουδιά πουλχερία
 πούλια πράβια πράγα πράσινο ακρωτήριο πράσσο πρέβεζα πρίσκιλλα πραξιθέα
 πρασά πρασιές πραστιό πριήνη πριγκιπάτο του λίχτενσταϊν πρινάρι προκοπία
 προκόπης προκόπιος προμηθέας προποντίδα προύσα πρωρέας πρωσία πρωτέας
 πρωτομαγιά πρωτοχρονιά πρόβιντενς πρόδρομος πρόσω ανατολή πρύμνη
 πσκοφ πτηνόν παραδείσιον πτολεμαΐδα πτολεμαία πτολεμαίος πτωχολέων
 πτώον πτώος πυθάρα πυθία πυθαγόρας πυθαγόρειο πυξίς πυργάκι πυρηναία πυρόγι
 πωλ πωλίνα πόβοα ντε βαρζίμ πόγραδετς πόζναν πόλα πόλη πόλυ πόμπλιος
 πόντος πόπη πόπλιος πόρος πόρτλαντ πόρτο πόρτο σάντο πότης πύλος πύργος
 πύρος πύρρος ράζγκραντ ράλλης ράνια ρέα ρέγκενσμπουργκ ρέθυμνο ρέματα ρέμος
 ρένα ρένος ρέντινγκ ρέσιτσα ρέυκιαβικ ρήγας ρήνη ρήνος ρία ρίγα ρίζος ρίμινι
 ρίο ντε τζανέιρο ρίτα ρίτσα ρίτσμοντ ρα ραβέννα ραγκουτσάρια ραιδεστός ρακλειά
 ραλλού ραμπάτ ρανταούτσι ραούλ ραπανούι ρας τζεμπέλ ραστ ρατζαστάν ραφ ραφ
 ραφαέλα ραφαήλ ραφαήλος ραφαηλία ραχήλ ρεβέκκα ρεγγίνα ρεζεδά ρεζεδάς ρεκένα
 ρεμς ρεν ρενάτος ρεταλουλέου ρεϋνιόν ρηγίνος ρηνιώ ρηνούλα ρητινό ριάντ ριέτι
 ριζούντα ριζοῦς ρικουζεντακάτα ριχάρδος ροβέρτος ρογήρος ρογκατσάρια ρογό
 ροδή ροδανός ροδιά ροδούλα ροδόη ροδόλφος ροδόπη ροζίτα ροζαλία ροκανιάρης
 ρομ ρομάν ρομπέν των δασών ρομπέρ ρομπέρτο ρομπόλα ρουάντα ρουέν ρουαγιάν
 ρουμάνα ρουμανία ρουμπίνα ρουμπίνη ρουμπαί ρουμόι ρουσέτος ρουτίλιο ρούκουνας
 ρούλης ρούμελη ρούσε ρούσος ρούσσος ρω ρωμέικο ρωμαίος ρωμανία ρωμανός
 ρωμυλία ρωμύλος ρωξάνη ρωσία ρωσοπόντιος ρόδης ρόδιος ρόδος ρόζα ρόζυ ρόης
 ρόμαν ρόουντ άιλαντ ρότερνταμ ρότσεστερ ρόχαρης ρώμη σάββας σάββατο σάγκα
 σάκα σάκης σάλεμ σάλονα σάλτσμπουργκ σάλωνα σάμος σάντα κρουθ ντε τενερίφε
 σάντα μαρία ντα φέιρα σάντα μαρία ντε λάμας σάντα ρόσα σάντα φε σάρα σάσα
 σάσσαρι σάτου μάρε σέβη σέβος σέρβα σέργιος σέριφος σέρρες σέτα σέτσε σέφκα
 σία σίβα σίβυλλα σίκινος σίλβα σίμος σίμων σίνες σίντι μπου ζαΰντ σίντρα σίσσυ
 σίτζες σίφνος σίφωνες σα φρατζίσκο σαΐκι σαΐτο σαίξπηρ σααρμπρύκεν σαβίνα
 σαββάτο σαββατιανό σαββούλα σαβιλιάνο σαβόνα σαγκάη σαιν λο σαιν ναζαίρ
 σαιν-μαλό σαιν-μπριέκ σαιντ μερ εγκλίζ σαιντ-ετιέν σαιντ-μωρ-ντε-τουραίν
 σακαβέμ σακατεπέκες σακραμέντο σαλέμι σαλέρνο σαλαβριάς σαλαμάνκα σαλαμίνα
 σαλαμπριάς σαλαχανάς σαλβάκη σαλούτσο σαλτσβούργο σαλό σαλόνικα σαλώμη σαμάνθα
 σαμανγκάν σαμεγκάβα σαμοθράκη σαμουήλ σαμπάουντια σαμπρίνα σαμψούντα σαμψοῦς
 σαμόα σαμόσατα σαν μάρκος σαν μαρίνο σαν ρέμο σαν σεβέρο σαν σεμπαστιάν
 σαν-τροπέ σαναά σανταρέμ σαντορίνη σαουδική αραβία σαπφώ σαράγεβο σαράγιεβο
 σαράντος σαρία σαραγόσα σαρακοστή σαραντούλα σαρβάτος σαρδηνία σαρλβίλ-μεζιέρ
 σαρωνικός σασέμπο σατουρνάλια σατσουμασεντάι σατωμπριάν σαχάρα σαϊέν σαϊκάι
 σβέριν σείριος σεβάστεια σεβίλλη σεβαστή σεβαστιανή σεβαστιανός σεβαστός
 σεγκόβια σεγόβια σεκούνδος σελήνη σελεστά σεληνιάς σελλοί σεμέλη σεμπόκου
 σενιγκάλια σεντάι σεούλ σεπτέμβρης σεπτέμβριος σεραπίων σεραφείμ σεραφείμ
 σερβία σερβία και μαυροβούνιο σεργής σερδανιόλα σερεμέτης σερμπούρ σερραία
 σεσίλ σεσίλια σετούμπαλ σεφέρης σεϋχέλλες σηκουάνας σημέλα σημίτης
 σημαία της γεωργίας σημαία της ελλάδας σητεία σιάμ σιάτιστα σιάτλ σιένα
 σιβηρία σιγγιτικός σιγκαπουριανή σιγκαπούρη σιδηρίτης σιδηρόκαστρο σιζουκουΐσι
 σιθωνία σικελία σικκίμ σικόκου σιλβάνα σιλβέστρος σιλβανή σιλβανός σιλγιάνα
 σιμαμπάρα σιμπέτσου σιμπίου σιμπλίκιος σιμπούσι σιμωνίδης σιμόγκο σιμώνη σινά
 σιογκάμα σιρακάβα σιροΐσι σισίνιος σιφ σιχαρί σκάλα σκίο σκαδό σκαλάδος
 σκαρλάτος σκεπόνι σκεύος σκιάθος σκιαδενή σκιαδόπουλο σκιπίων σκλαβοχώρι
 σκληρή σκορπιός σκοτία σκοτσέζα σκυθία σκυλλούρα σκυλοπνίχτης σκωτία σκωτσέζα
 σκόπια σκότος σκύλλα σκύρος σκώτος σλάτινα σλαβομακεδόνας σλοβακία σλοβενία
 σλουντέρνο σμίλη σμαράγδα σμαράγδω σμαραγδένια σμαραγδή σμαραγδία σμαρδάκι
 σμαρώ σμολένσκ σμύρνη σοβιετική ένωση σολιμάν σολολά σολομών σολτ λέικ σίτυ
 σομαλία σορέντο σορεζίνα σου σουαζιλάνδη σουβάλα σουβλίτσα σουδάν σουζάνα
 σουηδία σουκαγκάβα σουλίτσα σουλινάς σουλιώτισσα σουλμόνα σουλτάνα σουμάτρα
 σουμέριος σουμίτα σουναγκάβα σουράνα σουρινάμ σουσού σουτσεάβα σουτσιτεπέκες
 σοφία σοφίτσα σοφιάννα σοφιανός σοφοκλής σοφονίας σοφούλα σοφός άνθρωπος σούζα
 σούλα σούλη σούλι σούρε σούφι σπάιντερμαν σπάρτακος σπάρτη σπάτα σπέτζα
 σπήλιος σπεράδος σπετσοπούλα σπεύσιππος σπινατσόλα σπολέτο σποράδες
 σπυράκης σπυράκος σπυρέτος σπυρίδων σπυριδούλα σπόγγοι σπύρος σρι λάνκα
 στάμος στάρα ζαγόρα στάσα στέλα στέλιος στέλλα στέργιος στέφανος
 στέφανος νότιος στέφη στέφος σταγοί σταμάτα σταμάτης σταμάτιος σταμέλος
 σταματία σταματίνα σταμούλης στασινή στασινός στασούλα σταυράκης σταυριανή
 σταυρούλα σταυρούπολις σταυρωτό σταυρός σταυρός του νότου σταχτοπούτα σταύρος
 στελίδα στελλίτσα στενή στενιές στεργιανή στερεά ελλάδα στεφανής στεφανία
 στιβάρι στοκχόλμη στουτγκάρδη στράουσμπεργκ στράτος στραπουριές στρασβούργο
 στρατοκλής στρατονίκη στρατόνικος στροµαγερίτης στρυμονικός στρυμόνας στυλίδα
 στυλιανός στύγα συκιώτης συλβέστρος συμέλα συμεών συμφερόπολις συνετί
 συρία συρακούσες συριανοχώρι σφίντου γκεόργκε σφακτηρία σφαξ σχηματάρι
 σχοινούσσα σχόλη σωζάνα σωζόπολη σωκράτης σωσίπατρος σωσθένης σωτήρης σωτήριος
 σωτηρία σωτηρούλα σωφρονία σωφρόνης σωφρόνιος σόβα σόλων σόμα σόνια σόντριο
 σότια σότσι σόφη σόφια σύλβα σύλβια σύλβιος σύμη σύμπαν σύρα σύρος σύσκληπος
 σώζων σώτια σώτος τάγος τάκης τάκιτος τάκου τάμεσης τάνια τάραντας τάρας
 τάσος τάταρος τέλενδος τέλης τέμενη τέμπλος τέξας τέρψη τέτα τέτη τέως τήλος
 τίβολι τίγρης τίμαρχος τίμος τίμων τίμωνας τίρανα τίργκου ζίου τίργκου μούρες
 τίρυνς τίτα τίτη τίτος ταΰγετος ταβίρα ταβιθά ταγκάβα ταγκατζό τακέο τακέτα
 τακιζάβα τακικάβα τακσίμ ταλίν ταλαχάσι ταλιμπάν ταλμούδ ταμάνα ταμάρα
 ταμούρα ταμπάκικα ταμπάχανα τανζανία τανοχάτα ταξίμ ταξιάρχης ταράντο ταράσιος
 ταραμπάδος ταργκόβιστε ταρμπ ταρουμίζου ταρσία ταρσός τασία τασκένδη τασμανία
 ταταύλα τατζικιστάν τατιάνα τατιανή ταφάλια ταχάρ ταϊβάν ταϊλάνδη ταϊπέι
 ταύρος ταύρου ταώς τβερ τειρεσίας τεισίας τελαμών τενεσί τεοτιουακάν τεργέστη
 τερκς και κέικος τερπνή τερψιθέα τερψιχόρη τετάρτη τεταρτίδες τεχεράνη τζάδος
 τζένερο τζένη τζέφρι τζίνα τζακάρτα τζαμάικα τζανέτος τζανής τζανεριά
 τζαννέτος τζαννής τζαννετάκης τζαννετάκος τζαρκάντ τζενεριά τζιά τζιμπουτί
 τζιορντάνο μπρούνο τζιούρτζιου τζοάννα τζοβάνα τζοκόντα τζον κιτς τζουζέπε
 τζωρτζ μπέρναρ σω τζωρτζάκης τζόρτζια τζώνης τζώρτζης τηθύς τηλέμαχος
 τιμισοάρα τιμοκλής τιμολέων τιμοχάρης τιμόδημος τιμόθεος τιμόκριτος τιμόλαος
 τιντορέτο τιργκόβιστε τισσαφέρνης τιτάν τιτάνας τιτίκα τιτανίς τιτσιάνο
 τοβάντα τογιάμα τοκουσίμα τολέδο τομάρ τομακομάι τομσκ τοξότης τοπίκα τορίνο
 τορμπές τορρεμολίνος τορόντο τοσοδούλα τοτονικαπάν τοτόρι τοτός τουβαλού
 τουλάκι τουλίτσα τουλούζη τουλόν τουρ τουρκία τουρκαλβανός τουρκμενιστάν
 τουρκομυλωνάς τουρλίτης τοχόκου τούλα τούλτσεα τράπεζα τρέντο τρέντον τρίβερο
 τρίγωνον τρίγωνον νότιον τρίκαλα τρίπολη τρίπουρα τρίσω τρίτεια τρίτη
 τραπεζούντα τρασπάργα τραχώνι τραϊανούπολη τραϊανός τρεμεθούσια
 τριάδα τριάκοντα τριάντης τριαντάρος τριαντάφυλλη τριαντάφυλλος
 τριανταφυλλιά τρικαλινή τρικαλινός τρικκαίος τριμίθι τρινιντάντ και τομπάγκο
 τριπόταμος τρισεύγενη τροία τροιζηνία τρομελίν νήσος τροντσάνο βερτσελλέζε
 τρυγόνα τρυγώνα τρόπις τρόφα τρύφων τρύφωνας τσάρλεστον τσέστε τσίμπα
 τσαμπίκα τσαμπίκος τσαμπικία τσαντ τσαούσι τσατίσγκαρ τσεγιέν τσερνομπίλ
 τσεσμέ τσεχία τσεχοσλοβακία τσιέτι τσικαλαριό τσικιμούλα τσικνοπέμπτη
 τσικούγκο τσιμαλτενάνγκο τσιντσίλια τσιρίγο τσιτσέκα τσιτόσε τσου τσουγκάρου
 τσουκούμι τσουλούφι τσουσίμα τσούμπου τυνησία τυρ τυρί τυρινή τυφών τυφώνας
 τωβίας τόγκο τόκιο τόλης τόλο τόμε τόμης τόνγκα τόνο τόνυ τόσου τότα τύμβου
 τύνιδα τύριος τύρναβος τύρος τύχη τύχων τώνης τώνια υάκινθος υακίνθη υβόννη
 υγεία υγιεία υδροφόρια υδροχόος υεμένη υμεναίος υμηττός υπάτη υπάτιος υπαπαντή
 υπερέχιος υπομονή υπρ υψίδενδρο υψηλά υόρκη φάβιος φάλλικα φάνης φάνο φάνος
 φάρσαλα φάσα φάτα μοργκάνα φάτιμα φέλγκεϊρας φέλτρε φένια φέρμο φήλικας φήλιξ
 φίλιππος φίλων φίλωνας φίρμος φίτζι φαέθων φαέντσα φαήλος φαίακες φαίδρα
 φαίδων φαίη φαβάρα φαβιανός φαζάνο φαιδρός φαιναρέτη φαλατάδος φαμπριάνο
 φανή φανίτσα φαναράκια φανερωμένη φανουρία φανούλα φανούρης φανούριος φαντίνος
 φαραγγιανά φαρακλό φαραώ φαρκαδόνα φαρμακονήσι φαρμακούσα φαρμούθιος
 φαρυάμπ φασηλίδα φασουλής φαφούτης φαύνος φαύστα φαύστος φεβρουάλια
 φεβρωνία φειδίας φελισιανός φελλός φερενίκη φερεντίνο φεριάνα φερνάνα φερράρα
 φερρούκιος φερφούθη φερό χωριό φερόες φηλίκουλα φθιώτιδα φιγκέιρα ντα φους
 φιλάγριος φιλάδελφος φιλάρετος φιλέορτος φιλέρι φιλέταιρος φιλήμων φιλίνος
 φιλίσκος φιλίτσα φιλαδέλφεια φιλανδέζος φιλαρέτη φιλησία φιλιατρά φιλιππάκης
 φιλιππία φιλιππίνες φιλιππιανός φιλιππικός φιλιππούπολη φιλιώ φιλλανδία
 φιλογόνιος φιλοθέα φιλοθέη φιλοκράτης φιλοκτήμων φιλοκτήτης φιλομένα φιλομήλα
 φιλοποίμην φιλούμενος φιλόδαμος φιλόθεος φιλόκυπρος φιλόλαος φιλόσοφος
 φιλότερος φιλώτι φινεές φινλανδία φιουμιτσίνο φλάβιος φλάνδρα φλένσμπουρκ
 φλαμανδία φλαμανδός φλαμουρού φλανδρώ φλεβάρης φλοράλια φλωρέντιος φλωρέτα
 φλόγιος φλόκα φλόρες φλόριντα φλώρα φλώρινα φλώριος φλώρος φοίβη φοίβος
 φοίνικες φοίνιξ φοίνισσες φοινίκη φοινικούντα φολέγανδρος φολίνιο φονταινεμπλώ
 φορμέλο φοσακέσια φουκαγκάβα φουκουσίμα φουκουόκα φουκούι φουκούτσου φουντάι
 φουράνο φουρουντόνο φουρτουνάτος φουρτουνιανός φουσάνα φουτάμπα φουτζιγιάμα
 φούλα φούτζι φράγκα φράγκος φράιζινγκ φράιμπουργκ φράνκφορτ φρέυγια φρίγκα
 φραγκίσκα φραγκίσκη φραγκίσκος φραγκοκάστελλο φραγκούλης φρανκφούρτη φρανσουά
 φραντζέσκος φραντζής φρασκάτι φρατζέσκα φρειβούργο φρειδερίκη φρειδερίκος
 φροζινόνε φροσύνη φρουμέντιος φρουραί φρυγία φρόσω φρύνη φυλλιά φωκάς φωκίδα
 φωκαείς φωκείς φωκιανό φωξάνη φωστήριος φωτάκης φωτάς φωτανέστης φωτεινή
 φωτεινός φωτούλα φωφώ φόβος φόγιετ φόντι φόρμια φότζα φύλαξ φώκαια φώντας φώτα
 φώτιος φώτω φώφη φᾶσις χάβρη χάγη χάιδω χάλαρα χάλκη χάναου χάρης χάρις
 χάροντας χάρρισμπεργκ χάρτες χάρτφορντ χάρυβδη χάσια χάσκοβο χάτιβα χάχλακας
 χέρτσογκενρατ χίλια βρύση χίμαιρα χίος χίτα χαβάη χαιρέμων χαιρήμων χαιρώνεια
 χαλάπα χαλανδρίτσα χαλκί χαλκίδα χαλκίς χαλκα-μπουνάρ χαλκιάδα χαλκιδική
 χαμαιλέων χαμπάροφσκ χαν χανάβα χαναάν χαναμάκι χανιά χανς κρίστιαν άντερσεν
 χαρά χαράλαμπος χαρίδημος χαρίκλεια χαρίλαος χαρίσης χαρίσιος χαρίτος χαρίτων
 χαραλαμπία χαριτίνη χαρούλα χαρτούμ χατζησταμούληδες χατζιράδος χατσιμαντάι
 χατσιρόγκατα χαφούζ χαϊγιάν χαϊδελβέργη χαϊλμπρόν χελμάντ χεράτ
 χιγκασιναρούσε χιλή χιμάτσαλ πραντές χινοεμάτα χιονάτη χιονία χιούστον χιράντο
 χιρακάβα χιραϊζούμι χιροσάκι χιροσίμα χιρόνο χισάρια χιτογιόσι χιχόνα χιόκι
 χλωρίτης χλόη χοκκάιντο χονγκ κονγκ χονδρόμαυρο χονολουλού χοντ χοσρόης
 χουμίλια χουνεντοάρα χουτιάπα χοφγκάισμαρ χρήστος χρίστα χρίστος χρηστάκης
 χριστάδελφος χριστίνα χριστιάνα χριστιανός χριστουγεννιάτης χριστοφίλη
 χριστούγεννα χριστούλης χριστόδουλος χριστός χριστόφορος χρυσάνθη χρυσάφης
 χρυσή χρυσαυγή χρυσαφένια χρυσηίς χρυσηλιού χρυσοβαλάντης χρυσοβαλάντω
 χρυσοπηγή χρυσοσθένης χρυσούλα χρυσούλη χρυσούπολη χρυστάλλα χρυσόστομος χρύσα
 χρύσπα χρύσω χυούγκα χόμπιτ χόνσου χώνες χώρα ψάθα ψάθη ψέριμος ψαθάδες
 ψαθάκι ψαθάκια ψαθοτόπι ψαθοχώρι ψαλίδα ψαλίδι ψαμμήτιχος ψαραντώνης ψαχνά
 ψηλορείτης ψυχικό ψυχώ ψωραροί ψόης ωβήδ ωγυγία ωγύγης ωκεανία ωκεανός ωκυπέτη
 ωραία ωραία κοιμωμένη ωραία πύλη ωραιάνθη ωραιοζήλη ωραιόκαστρο ωρεοί ωριγένης
 ωρολόγιον ωρωπός ωσηέ όγκα όγκατα όγκι όθρυς όθων όιτα όλγα όλιβερ όλυμπος
 όμουρα όμπιδος όνειρος όνο όρεγκον όσακι όσατο όσκαρ όσλο όταμα ότσου όφενμπαχ
 όχιρα ύδρα ύδρος ύψιστος ώλενος ώρες ώρχους ώστιν ἀλεξανδρούπολις ἀμαλιούπολις
""".split())



# coding: utf8
from __future__ import unicode_literals

import plac
from pathlib import Path

from ._messages import Messages
from ..compat import symlink_to, path2str
from ..util import prints
from .. import util


@plac.annotations(
    origin=("package name or local path to model", "positional", None, str),
    link_name=("name of shortuct link to create", "positional", None, str),
    force=("force overwriting of existing link", "flag", "f", bool))
def link(origin, link_name, force=False, model_path=None):
    """
    Create a symlink for models within the spacy/data directory. Accepts
    either the name of a pip package, or the local path to the model data
    directory. Linking models allows loading them via spacy.load(link_name).
    """
    if util.is_package(origin):
        model_path = util.get_package_path(origin)
    else:
        model_path = Path(origin) if model_path is None else Path(model_path)
    if not model_path.exists():
        prints(Messages.M009.format(path=path2str(model_path)),
               title=Messages.M008, exits=1)
    data_path = util.get_data_path()
    if not data_path or not data_path.exists():
        spacy_loc = Path(__file__).parent.parent
        prints(Messages.M011, spacy_loc, title=Messages.M010, exits=1)
    link_path = util.get_data_path() / link_name
    if link_path.is_symlink() and not force:
        prints(Messages.M013, title=Messages.M012.format(name=link_name),
               exits=1)
    elif link_path.is_symlink():  # does a symlink exist?
        # NB: It's important to check for is_symlink here and not for exists,
        # because invalid/outdated symlinks would return False otherwise.
        link_path.unlink()
    elif link_path.exists(): # does it exist otherwise?
        # NB: Check this last because valid symlinks also "exist".
        prints(Messages.M015, link_path,
               title=Messages.M014.format(name=link_name), exits=1)
    msg = "%s --> %s" % (path2str(model_path), path2str(link_path))
    try:
        symlink_to(link_path, model_path)
    except:
        # This is quite dirty, but just making sure other errors are caught.
        prints(Messages.M017, msg, title=Messages.M016.format(name=link_name))
        raise
    prints(msg, Messages.M019.format(name=link_name), title=Messages.M018)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import pytest


def test_tokenizer_handles_long_text(el_tokenizer):
    text = """Η Ελλάδα (παλαιότερα Ελλάς), επίσημα γνωστή ως Ελληνική Δημοκρατία,\
    είναι χώρα της νοτιοανατολικής Ευρώπης στο νοτιότερο άκρο της Βαλκανικής χερσονήσου.\
    Συνορεύει στα βορειοδυτικά με την Αλβανία, στα βόρεια με την πρώην\
    Γιουγκοσλαβική Δημοκρατία της Μακεδονίας και τη Βουλγαρία και στα βορειοανατολικά με την Τουρκία."""
    tokens = el_tokenizer(text)
    assert len(tokens) == 54


@pytest.mark.parametrize('text,length',[
    ("Διοικητικά η Ελλάδα διαιρείται σε 13 Περιφέρειες.", 8),
    ("Η εκπαίδευση στην Ελλάδα χωρίζεται κυρίως σε τρία επίπεδα.", 10),
    ("Η Ελλάδα είναι μία από τις χώρες της Ευρωπαϊκής Ένωσης (ΕΕ) που διαθέτει σηµαντικό ορυκτό πλούτο.", 19),
    ("Η ναυτιλία αποτέλεσε ένα σημαντικό στοιχείο της Ελληνικής οικονομικής δραστηριότητας από τα αρχαία χρόνια.", 15),
    ("Η Ελλάδα είναι μέλος σε αρκετούς διεθνείς οργανισμούς.", 9)])
def test_tokenizer_handles_cnts(el_tokenizer,text, length):
    tokens = el_tokenizer(text)
    assert len(tokens) == length


# coding: utf8
from __future__ import unicode_literals

import plac
import math
from tqdm import tqdm
import numpy
from ast import literal_eval
from pathlib import Path
from preshed.counter import PreshCounter
import tarfile
import gzip
import zipfile

from ._messages import Messages
from ..vectors import Vectors
from ..errors import Errors, Warnings, user_warning
from ..util import prints, ensure_path, get_lang_class

try:
    import ftfy
except ImportError:
    ftfy = None


@plac.annotations(
    lang=("model language", "positional", None, str),
    output_dir=("model output directory", "positional", None, Path),
    freqs_loc=("location of words frequencies file", "positional", None, Path),
    clusters_loc=("optional: location of brown clusters data",
                  "option", "c", str),
    vectors_loc=("optional: location of vectors file in Word2Vec format "
                 "(either as .txt or zipped as .zip or .tar.gz)", "option",
                 "v", str),
    prune_vectors=("optional: number of vectors to prune to",
                   "option", "V", int)
)
def init_model(lang, output_dir, freqs_loc=None, clusters_loc=None,
               vectors_loc=None, prune_vectors=-1):
    """
    Create a new model from raw data, like word frequencies, Brown clusters
    and word vectors.
    """
    if freqs_loc is not None and not freqs_loc.exists():
        prints(freqs_loc, title=Messages.M037, exits=1)
    clusters_loc = ensure_path(clusters_loc)
    vectors_loc = ensure_path(vectors_loc)
    probs, oov_prob = read_freqs(freqs_loc) if freqs_loc is not None else ({}, -20)
    vectors_data, vector_keys = read_vectors(vectors_loc) if vectors_loc else (None, None)
    clusters = read_clusters(clusters_loc) if clusters_loc else {}
    nlp = create_model(lang, probs, oov_prob, clusters, vectors_data, vector_keys, prune_vectors)
    if not output_dir.exists():
        output_dir.mkdir()
    nlp.to_disk(output_dir)
    return nlp

def open_file(loc):
    '''Handle .gz, .tar.gz or unzipped files'''
    loc = ensure_path(loc)
    print("Open loc")
    if tarfile.is_tarfile(str(loc)):
        return tarfile.open(str(loc), 'r:gz')
    elif loc.parts[-1].endswith('gz'):
        return (line.decode('utf8') for line in gzip.open(str(loc), 'r'))
    elif loc.parts[-1].endswith('zip'):
        zip_file = zipfile.ZipFile(str(loc))
        names = zip_file.namelist()
        file_ = zip_file.open(names[0])
        return (line.decode('utf8') for line in file_)
    else:
        return loc.open('r', encoding='utf8')

def create_model(lang, probs, oov_prob, clusters, vectors_data, vector_keys, prune_vectors):
    print("Creating model...")
    lang_class = get_lang_class(lang)
    nlp = lang_class()
    for lexeme in nlp.vocab:
        lexeme.rank = 0
    lex_added = 0
    for i, (word, prob) in enumerate(tqdm(sorted(probs.items(), key=lambda item: item[1], reverse=True))):
        lexeme = nlp.vocab[word]
        lexeme.rank = i
        lexeme.prob = prob
        lexeme.is_oov = False
        # Decode as a little-endian string, so that we can do & 15 to get
        # the first 4 bits. See _parse_features.pyx
        if word in clusters:
            lexeme.cluster = int(clusters[word][::-1], 2)
        else:
            lexeme.cluster = 0
        lex_added += 1
    nlp.vocab.cfg.update({'oov_prob': oov_prob})
    for word in vector_keys:
        if word not in nlp.vocab:
            lexeme = nlp.vocab[word]
            lexeme.is_oov = False
            lex_added += 1
    if len(vectors_data):
        nlp.vocab.vectors = Vectors(data=vectors_data, keys=vector_keys)
    if prune_vectors >= 1:
        nlp.vocab.prune_vectors(prune_vectors)
    vec_added = len(nlp.vocab.vectors)
    prints(Messages.M039.format(entries=lex_added, vectors=vec_added),
           title=Messages.M038)
    return nlp


def read_vectors(vectors_loc):
    print("Reading vectors from %s" % vectors_loc)
    f = open_file(vectors_loc)
    shape = tuple(int(size) for size in next(f).split())
    vectors_data = numpy.zeros(shape=shape, dtype='f')
    vectors_keys = []
    for i, line in enumerate(tqdm(f)):
        line = line.rstrip()
        pieces = line.rsplit(' ', vectors_data.shape[1]+1)
        word = pieces.pop(0)
        if len(pieces) != vectors_data.shape[1]:
            raise ValueError(Errors.E094.format(line_num=i, loc=vectors_loc))
        vectors_data[i] = numpy.asarray(pieces, dtype='f')
        vectors_keys.append(word)
    return vectors_data, vectors_keys


def read_freqs(freqs_loc, max_length=100, min_doc_freq=5, min_freq=50):
    print("Counting frequencies...")
    counts = PreshCounter()
    total = 0
    with freqs_loc.open() as f:
        for i, line in enumerate(f):
            freq, doc_freq, key = line.rstrip().split('\t', 2)
            freq = int(freq)
            counts.inc(i + 1, freq)
            total += freq
    counts.smooth()
    log_total = math.log(total)
    probs = {}
    with freqs_loc.open() as f:
        for line in tqdm(f):
            freq, doc_freq, key = line.rstrip().split('\t', 2)
            doc_freq = int(doc_freq)
            freq = int(freq)
            if doc_freq >= min_doc_freq and freq >= min_freq and len(key) < max_length:
                word = literal_eval(key)
                smooth_count = counts.smoother(int(freq))
                probs[word] = math.log(smooth_count) - log_total
    oov_prob = math.log(counts.smoother(0)) - log_total
    return probs, oov_prob


def read_clusters(clusters_loc):
    print("Reading clusters...")
    clusters = {}
    if ftfy is None:
        user_warning(Warnings.W004)
    with clusters_loc.open() as f:
        for line in tqdm(f):
            try:
                cluster, word, freq = line.split()
                if ftfy is not None:
                    word = ftfy.fix_text(word)
            except ValueError:
                continue
            # If the clusterer has only seen the word a few times, its
            # cluster is unreliable.
            if int(freq) >= 3:
                clusters[word] = cluster
            else:
                clusters[word] = '0'
    # Expand clusters with re-casing
    for word, cluster in list(clusters.items()):
        if word.lower() not in clusters:
            clusters[word.lower()] = cluster
        if word.title() not in clusters:
            clusters[word.title()] = cluster
        if word.upper() not in clusters:
            clusters[word.upper()] = cluster
    return clusters



'''Test comparison against None doesn't cause segfault'''
from __future__ import unicode_literals

from ...tokens import Doc
from ...vocab import Vocab

def test_issue1757():
    doc = Doc(Vocab(), words=['a', 'b', 'c'])
    assert not doc[0] < None
    assert not doc[0] == None
    assert doc[0] >= None
    span = doc[:2]
    assert not span < None
    assert not span == None
    assert span >= None
    lex = doc.vocab['a']
    assert not lex == None
    assert not lex < None
